import pydash
import pytest
from marshmallow import ValidationError

from azure.ai.ml._utils.utils import load_yaml
from azure.ai.ml.entities._load_functions import load_schedule

from .._util import _SCHEDULE_TIMEOUT_SECOND


@pytest.mark.timeout(_SCHEDULE_TIMEOUT_SECOND)
@pytest.mark.unittest
@pytest.mark.pipeline_test
class TestScheduleSchema:
    def test_load_cron_schedule_with_file_reference(self):
        test_path = "./tests/test_configs/schedule/hello_cron_schedule_with_file_reference.yml"
        schedule = load_schedule(test_path)
        expected_dict = {
            "name": "weekly_retrain_2022_cron_file",
            "description": "a weekly retrain schedule",
            "display_name": "weekly retrain schedule",
            "trigger": {
                "start_time": "2022-03-10T10:15:00",
                "end_time": "2022-06-10T10:15:00",
                "time_zone": "Pacific Standard Time",
                "type": "cron",
                "expression": "15 10 * * 1",
            },
            "create_job": {
                "display_name": "hello_pipeline_abc",
                "compute": "azureml:cpu-cluster",
                "type": "pipeline",
                "inputs": {"hello_string_top_level_input": {"path": "${{name}}"}},
                "jobs": {
                    "a": {
                        "inputs": {"hello_string": {"path": "${{parent.inputs.hello_string_top_level_input}}"}},
                        "component": {
                            "name": "azureml_anonymous",
                            "version": "1",
                            "is_deterministic": True,
                            "inputs": {"hello_string": {"type": "string"}},
                            "type": "command",
                            "command": "echo hello ${{inputs.hello_string}}",
                            "environment": "azureml:AzureML-sklearn-1.0-ubuntu20.04-py38-cpu@latest",
                        },
                        "type": "command",
                    },
                    "b": {
                        "component": {
                            "name": "azureml_anonymous",
                            "version": "1",
                            "is_deterministic": True,
                            "outputs": {"world_output": {"type": "uri_folder"}},
                            "type": "command",
                            "command": 'echo "world" >> ${{outputs.world_output}}/world.txt',
                            "environment": "azureml:AzureML-sklearn-1.0-ubuntu20.04-py38-cpu@latest",
                        },
                        "type": "command",
                    },
                    "c": {
                        "inputs": {"world_input": {"path": "${{parent.jobs.b.outputs.world_output}}"}},
                        "component": {
                            "name": "azureml_anonymous",
                            "version": "1",
                            "is_deterministic": True,
                            "inputs": {"world_input": {"type": "uri_folder"}},
                            "type": "command",
                            "command": "echo ${{inputs.world_input}}/world.txt",
                            "environment": "azureml:AzureML-sklearn-1.0-ubuntu20.04-py38-cpu@latest",
                        },
                        "type": "command",
                    },
                },
            },
        }
        assert schedule._to_dict() == expected_dict

    def test_load_cron_schedule_with_job_updates(self):
        test_path = "./tests/test_configs/schedule/hello_cron_schedule_with_job_updates.yml"
        yaml_obj = load_yaml(test_path)
        expected_updates = yaml_obj["create_job"]
        expected_updates.pop("job")
        # Workaround the binding shape changes
        expected_updates["inputs"]["hello_string_top_level_input"] = {"path": "${{creation_context.trigger_time}}"}
        scheduled_job = load_schedule(test_path).create_job
        actual_dict = scheduled_job._to_dict()
        for key, val in expected_updates.items():
            assert actual_dict[key] == val

    def test_load_cron_schedule_with_arm_id(self):
        test_path = "./tests/test_configs/schedule/hello_cron_schedule_with_arm_id.yml"
        schedule = load_schedule(test_path)
        expected_dict = {
            "name": "weekly_retrain_2022_cron_arm",
            "description": "a weekly retrain schedule",
            "display_name": "weekly retrain schedule",
            "trigger": {"time_zone": "UTC", "type": "cron", "expression": "15 10 * * 1"},
            "create_job": "azureml:/subscriptions/d511f82f-71ba-49a4-8233-d7be8a3650f4/resourceGroups/RLTesting/providers/Microsoft.MachineLearningServices/workspaces/AnkitWS/jobs/test_617704734544",
        }
        assert schedule._to_dict() == expected_dict

    def test_load_cron_schedule_with_arm_id_and_updates(self):
        test_path = "./tests/test_configs/schedule/hello_cron_schedule_with_arm_id_and_updates.yml"
        schedule = load_schedule(test_path)
        expected_dict = {
            "create_job": {
                "experiment_name": "schedule_test_exp",
                "id": "azureml:/subscriptions/d511f82f-71ba-49a4-8233-d7be8a3650f4/resourceGroups/RLTesting/providers/Microsoft.MachineLearningServices/workspaces/AnkitWS/jobs/test_617704734544",
                "inputs": {"hello_string_top_level_input": {"path": "${{name}}"}},
                "settings": {"continue_on_step_failure": True, "default_compute": "azureml:cpu-cluster"},
                "type": "pipeline",
            },
            "name": "weekly_retrain_2022_cron_arm_updates",
            "trigger": {
                "expression": "15 10 * * 1",
                "start_time": "2022-03-10T10:15:00",
                "time_zone": "UTC",
                "type": "cron",
            },
        }
        assert schedule._to_dict() == expected_dict

    def test_load_recurrence_schedule_no_pattern(self):
        test_path = "./tests/test_configs/schedule/hello_recurrence_schedule_no_pattern.yml"
        schedule = load_schedule(test_path)
        yaml_obj = load_yaml(test_path)
        expected_trigger_dict = yaml_obj["trigger"]
        # Append empty pattern
        assert schedule not in expected_trigger_dict
        actual_trigger_dict = schedule._to_dict()["trigger"]
        # Remove emtpy key 'schedule': {'hours': [], 'minutes': []}
        actual_trigger_dict = pydash.omit(actual_trigger_dict, ["schedule"])
        assert actual_trigger_dict == expected_trigger_dict

    def test_load_recurrence_schedule_with_pattern(self):
        test_path = "./tests/test_configs/schedule/hello_recurrence_schedule_with_pattern.yml"
        schedule = load_schedule(test_path)
        yaml_obj = load_yaml(test_path)
        expected_trigger_dict = {
            "frequency": "week",
            "interval": 1,
            "schedule": {"hours": 10, "minutes": 15, "week_days": "monday"},
            "start_time": "2022-05-10T10:15:00",
            "time_zone": "Pacific Standard Time",
            "type": "recurrence",
        }
        assert schedule._to_dict()["trigger"] == expected_trigger_dict
        expected_updates = yaml_obj["create_job"]
        expected_updates.pop("job")
        scheduled_job = load_schedule(test_path).create_job
        actual_dict = scheduled_job._to_dict()
        for key, val in expected_updates.items():
            assert actual_dict[key] == val

    def test_load_invalid_schedule_missing_type(self):
        test_path = "./tests/test_configs/schedule/invalid/hello_cron_schedule_with_arm_id_no_type.yml"
        with pytest.raises(ValidationError) as e:
            load_schedule(test_path)
        assert "'type' must be specified when scheduling a remote job with updates." in e.value.messages[0]



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=unused-argument

import logging
from typing import Any

from marshmallow import fields, post_load

from azure.ai.ml._schema.core.schema import PatchedSchemaMeta

module_logger = logging.getLogger(__name__)


class RequestSettingsSchema(metaclass=PatchedSchemaMeta):
    request_timeout_ms = fields.Int(required=False)
    max_concurrent_requests_per_instance = fields.Int(required=False)
    max_queue_wait_ms = fields.Int(required=False)

    @post_load
    def make(self, data: Any, **kwargs: Any) -> Any:
        from azure.ai.ml.entities import OnlineRequestSettings

        return OnlineRequestSettings(**data)



from unittest.mock import Mock, patch

import pytest

from azure.ai.ml._scope_dependent_operations import OperationScope
from azure.ai.ml.constants._common import AzureMLResourceType
from azure.ai.ml.operations import OnlineDeploymentOperations


@pytest.fixture
def mock_endpoint_operations(
    mock_workspace_scope: OperationScope,
    mock_aml_services_2022_02_01_preview: Mock,
    mock_aml_services_2020_09_01_dataplanepreview: Mock,
    mock_machinelearning_client: Mock,
    mock_environment_operations: Mock,
    mock_model_operations: Mock,
    mock_code_assets_operations: Mock,
    mock_data_operations: Mock,
    mock_local_endpoint_helper: Mock,
) -> OnlineDeploymentOperations:
    mock_machinelearning_client._operation_container.add(AzureMLResourceType.CODE, mock_code_assets_operations)
    mock_machinelearning_client._operation_container.add(AzureMLResourceType.MODEL, mock_code_assets_operations)
    mock_machinelearning_client._operation_container.add(AzureMLResourceType.ENVIRONMENT, mock_code_assets_operations)
    mock_machinelearning_client._operation_container.add(AzureMLResourceType.DATA, mock_data_operations)

    yield OnlineDeploymentOperations(
        operation_scope=mock_workspace_scope,
        service_client_02_2022_preview=mock_aml_services_2022_02_01_preview,
        service_client_09_2020_dataplanepreview=mock_aml_services_2020_09_01_dataplanepreview,
        all_operations=mock_machinelearning_client._operation_container,
        local_endpoint_helper=mock_local_endpoint_helper,
    )


@pytest.mark.unittest
@pytest.mark.production_experiences_test
class TestOnlineEndpointsOperations:
    def test_mock_endpoint_operation(mock_endpoint_operations):
        assert mock_endpoint_operations is not None



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from marshmallow import fields

from azure.ai.ml._schema.core.schema import PatchedSchemaMeta
from azure.ai.ml._schema.core.fields import DataBindingStr, UnionField


class RetrySettingsSchema(metaclass=PatchedSchemaMeta):
    timeout = UnionField([fields.Int(), DataBindingStr])
    max_retries = UnionField([fields.Int(), DataBindingStr])



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
import abc
from typing import Any


class AsyncContextManager(abc.ABC):
    @abc.abstractmethod
    async def close(self) -> None:
        pass

    async def __aenter__(self) -> "AsyncContextManager":
        return self

    async def __aexit__(self, *args: Any) -> None:
        await self.close()


__all__ = ["AsyncContextManager"]



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------
"""This module will load mlflow model and do prediction."""

import argparse
import os

from mlflow.sklearn import load_model

MODEL_NAME = "iris_model"


def init():
    print("Environment variables start ****")
    for key, val in os.environ.items():
        print(key, val)
    print("Environment variables end ****")

    parser = argparse.ArgumentParser(allow_abbrev=False, description="ParallelRunStep Agent")
    parser.add_argument("--model", type=str, default=0)
    args, _ = parser.parse_known_args()

    model_path = args.model + "/" + MODEL_NAME
    global iris_model

    iris_model = load_model(model_path)


def run(input_data):
    num_rows, num_cols = input_data.shape
    pred = iris_model.predict(input_data).reshape((num_rows, 1))

    # cleanup output
    result = input_data.drop(input_data.columns[4:], axis=1)
    result["variety"] = pred

    return result



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

try:
    from ._models_py3 import AKS
    from ._models_py3 import AKSSchema
    from ._models_py3 import AKSSchemaProperties
    from ._models_py3 import AccessKeyAuthTypeWorkspaceConnectionProperties
    from ._models_py3 import AccountKeyDatastoreCredentials
    from ._models_py3 import AccountKeyDatastoreSecrets
    from ._models_py3 import AcrDetails
    from ._models_py3 import AksComputeSecrets
    from ._models_py3 import AksComputeSecretsProperties
    from ._models_py3 import AksNetworkingConfiguration
    from ._models_py3 import AllNodes
    from ._models_py3 import AmlCompute
    from ._models_py3 import AmlComputeNodeInformation
    from ._models_py3 import AmlComputeNodesInformation
    from ._models_py3 import AmlComputeProperties
    from ._models_py3 import AmlComputeSchema
    from ._models_py3 import AmlOperation
    from ._models_py3 import AmlOperationDisplay
    from ._models_py3 import AmlOperationListResult
    from ._models_py3 import AmlToken
    from ._models_py3 import AmlUserFeature
    from ._models_py3 import ArmResourceId
    from ._models_py3 import AssetBase
    from ._models_py3 import AssetContainer
    from ._models_py3 import AssetJobInput
    from ._models_py3 import AssetJobOutput
    from ._models_py3 import AssetReferenceBase
    from ._models_py3 import AssignedUser
    from ._models_py3 import AutoForecastHorizon
    from ._models_py3 import AutoMLJob
    from ._models_py3 import AutoMLVertical
    from ._models_py3 import AutoNCrossValidations
    from ._models_py3 import AutoPauseProperties
    from ._models_py3 import AutoScaleProperties
    from ._models_py3 import AutoSeasonality
    from ._models_py3 import AutoTargetLags
    from ._models_py3 import AutoTargetRollingWindowSize
    from ._models_py3 import AutologgerSettings
    from ._models_py3 import AzureBlobDatastore
    from ._models_py3 import AzureDataLakeGen1Datastore
    from ._models_py3 import AzureDataLakeGen2Datastore
    from ._models_py3 import AzureDatastore
    from ._models_py3 import AzureFileDatastore
    from ._models_py3 import AzureMLBatchInferencingServer
    from ._models_py3 import AzureMLOnlineInferencingServer
    from ._models_py3 import BanditPolicy
    from ._models_py3 import BaseEnvironmentId
    from ._models_py3 import BaseEnvironmentSource
    from ._models_py3 import BatchDeployment
    from ._models_py3 import BatchDeploymentProperties
    from ._models_py3 import BatchDeploymentPropertiesAutoGenerated
    from ._models_py3 import BatchDeploymentTrackedResourceArmPaginatedResult
    from ._models_py3 import BatchEndpoint
    from ._models_py3 import BatchEndpointDefaults
    from ._models_py3 import BatchEndpointProperties
    from ._models_py3 import BatchEndpointTrackedResourceArmPaginatedResult
    from ._models_py3 import BatchPipelineComponentDeploymentProperties
    from ._models_py3 import BatchRetrySettings
    from ._models_py3 import BayesianSamplingAlgorithm
    from ._models_py3 import BindOptions
    from ._models_py3 import BuildContext
    from ._models_py3 import CertificateDatastoreCredentials
    from ._models_py3 import CertificateDatastoreSecrets
    from ._models_py3 import Classification
    from ._models_py3 import ClassificationTrainingSettings
    from ._models_py3 import ClusterUpdateParameters
    from ._models_py3 import CocoExportSummary
    from ._models_py3 import CodeConfiguration
    from ._models_py3 import CodeContainer
    from ._models_py3 import CodeContainerProperties
    from ._models_py3 import CodeContainerResourceArmPaginatedResult
    from ._models_py3 import CodeVersion
    from ._models_py3 import CodeVersionProperties
    from ._models_py3 import CodeVersionResourceArmPaginatedResult
    from ._models_py3 import ColumnTransformer
    from ._models_py3 import CommandJob
    from ._models_py3 import CommandJobLimits
    from ._models_py3 import ComponentContainer
    from ._models_py3 import ComponentContainerProperties
    from ._models_py3 import ComponentContainerResourceArmPaginatedResult
    from ._models_py3 import ComponentVersion
    from ._models_py3 import ComponentVersionProperties
    from ._models_py3 import ComponentVersionResourceArmPaginatedResult
    from ._models_py3 import Compute
    from ._models_py3 import ComputeInstance
    from ._models_py3 import ComputeInstanceApplication
    from ._models_py3 import ComputeInstanceAutologgerSettings
    from ._models_py3 import ComputeInstanceConnectivityEndpoints
    from ._models_py3 import ComputeInstanceContainer
    from ._models_py3 import ComputeInstanceCreatedBy
    from ._models_py3 import ComputeInstanceDataDisk
    from ._models_py3 import ComputeInstanceDataMount
    from ._models_py3 import ComputeInstanceEnvironmentInfo
    from ._models_py3 import ComputeInstanceLastOperation
    from ._models_py3 import ComputeInstanceProperties
    from ._models_py3 import ComputeInstanceSchema
    from ._models_py3 import ComputeInstanceSshSettings
    from ._models_py3 import ComputeInstanceVersion
    from ._models_py3 import ComputeResource
    from ._models_py3 import ComputeResourceSchema
    from ._models_py3 import ComputeSchedules
    from ._models_py3 import ComputeSecrets
    from ._models_py3 import ComputeStartStopSchedule
    from ._models_py3 import ContainerResourceRequirements
    from ._models_py3 import ContainerResourceSettings
    from ._models_py3 import CosmosDbSettings
    from ._models_py3 import CronTrigger
    from ._models_py3 import CsvExportSummary
    from ._models_py3 import CustomForecastHorizon
    from ._models_py3 import CustomInferencingServer
    from ._models_py3 import CustomModelJobInput
    from ._models_py3 import CustomModelJobOutput
    from ._models_py3 import CustomNCrossValidations
    from ._models_py3 import CustomSeasonality
    from ._models_py3 import CustomService
    from ._models_py3 import CustomTargetLags
    from ._models_py3 import CustomTargetRollingWindowSize
    from ._models_py3 import DataContainer
    from ._models_py3 import DataContainerProperties
    from ._models_py3 import DataContainerResourceArmPaginatedResult
    from ._models_py3 import DataFactory
    from ._models_py3 import DataLakeAnalytics
    from ._models_py3 import DataLakeAnalyticsSchema
    from ._models_py3 import DataLakeAnalyticsSchemaProperties
    from ._models_py3 import DataPathAssetReference
    from ._models_py3 import DataVersionBase
    from ._models_py3 import DataVersionBaseProperties
    from ._models_py3 import DataVersionBaseResourceArmPaginatedResult
    from ._models_py3 import Databricks
    from ._models_py3 import DatabricksComputeSecrets
    from ._models_py3 import DatabricksComputeSecretsProperties
    from ._models_py3 import DatabricksProperties
    from ._models_py3 import DatabricksSchema
    from ._models_py3 import DatasetExportSummary
    from ._models_py3 import Datastore
    from ._models_py3 import DatastoreCredentials
    from ._models_py3 import DatastoreProperties
    from ._models_py3 import DatastoreResourceArmPaginatedResult
    from ._models_py3 import DatastoreSecrets
    from ._models_py3 import DefaultScaleSettings
    from ._models_py3 import DeploymentLogs
    from ._models_py3 import DeploymentLogsRequest
    from ._models_py3 import DeploymentResourceConfiguration
    from ._models_py3 import DiagnoseRequestProperties
    from ._models_py3 import DiagnoseResponseResult
    from ._models_py3 import DiagnoseResponseResultValue
    from ._models_py3 import DiagnoseResult
    from ._models_py3 import DiagnoseWorkspaceParameters
    from ._models_py3 import DistributionConfiguration
    from ._models_py3 import Docker
    from ._models_py3 import EarlyTerminationPolicy
    from ._models_py3 import EncryptionKeyVaultProperties
    from ._models_py3 import EncryptionKeyVaultUpdateProperties
    from ._models_py3 import EncryptionProperty
    from ._models_py3 import EncryptionUpdateProperties
    from ._models_py3 import Endpoint
    from ._models_py3 import EndpointAuthKeys
    from ._models_py3 import EndpointAuthToken
    from ._models_py3 import EndpointDeploymentPropertiesBase
    from ._models_py3 import EndpointPropertiesBase
    from ._models_py3 import EndpointScheduleAction
    from ._models_py3 import EnvironmentContainer
    from ._models_py3 import EnvironmentContainerProperties
    from ._models_py3 import EnvironmentContainerResourceArmPaginatedResult
    from ._models_py3 import EnvironmentVariable
    from ._models_py3 import EnvironmentVersion
    from ._models_py3 import EnvironmentVersionProperties
    from ._models_py3 import EnvironmentVersionResourceArmPaginatedResult
    from ._models_py3 import ErrorAdditionalInfo
    from ._models_py3 import ErrorDetail
    from ._models_py3 import ErrorResponse
    from ._models_py3 import EstimatedVMPrice
    from ._models_py3 import EstimatedVMPrices
    from ._models_py3 import ExportSummary
    from ._models_py3 import ExternalFQDNResponse
    from ._models_py3 import FQDNEndpoint
    from ._models_py3 import FQDNEndpointDetail
    from ._models_py3 import FQDNEndpoints
    from ._models_py3 import FQDNEndpointsProperties
    from ._models_py3 import Feature
    from ._models_py3 import FeatureArmPaginatedResult
    from ._models_py3 import FeatureWindow
    from ._models_py3 import FeaturesetContainer
    from ._models_py3 import FeaturesetContainerProperties
    from ._models_py3 import FeaturesetContainerResourceArmPaginatedResult
    from ._models_py3 import FeaturesetJob
    from ._models_py3 import FeaturesetJobArmPaginatedResult
    from ._models_py3 import FeaturesetSpecification
    from ._models_py3 import FeaturesetVersion
    from ._models_py3 import FeaturesetVersionBackfillRequest
    from ._models_py3 import FeaturesetVersionBackfillResponse
    from ._models_py3 import FeaturesetVersionProperties
    from ._models_py3 import FeaturesetVersionResourceArmPaginatedResult
    from ._models_py3 import FeaturestoreEntityContainer
    from ._models_py3 import FeaturestoreEntityContainerProperties
    from ._models_py3 import FeaturestoreEntityContainerResourceArmPaginatedResult
    from ._models_py3 import FeaturestoreEntityVersion
    from ._models_py3 import FeaturestoreEntityVersionProperties
    from ._models_py3 import FeaturestoreEntityVersionResourceArmPaginatedResult
    from ._models_py3 import FeaturizationSettings
    from ._models_py3 import FlavorData
    from ._models_py3 import ForecastHorizon
    from ._models_py3 import Forecasting
    from ._models_py3 import ForecastingSettings
    from ._models_py3 import ForecastingTrainingSettings
    from ._models_py3 import GetFeatureRequest
    from ._models_py3 import GridSamplingAlgorithm
    from ._models_py3 import HDInsight
    from ._models_py3 import HDInsightProperties
    from ._models_py3 import HDInsightSchema
    from ._models_py3 import HdfsDatastore
    from ._models_py3 import IdAssetReference
    from ._models_py3 import IdentityConfiguration
    from ._models_py3 import IdentityForCmk
    from ._models_py3 import IdleShutdownSetting
    from ._models_py3 import Image
    from ._models_py3 import ImageClassification
    from ._models_py3 import ImageClassificationBase
    from ._models_py3 import ImageClassificationMultilabel
    from ._models_py3 import ImageInstanceSegmentation
    from ._models_py3 import ImageLimitSettings
    from ._models_py3 import ImageMetadata
    from ._models_py3 import ImageModelDistributionSettings
    from ._models_py3 import ImageModelDistributionSettingsClassification
    from ._models_py3 import ImageModelDistributionSettingsObjectDetection
    from ._models_py3 import ImageModelSettings
    from ._models_py3 import ImageModelSettingsClassification
    from ._models_py3 import ImageModelSettingsObjectDetection
    from ._models_py3 import ImageObjectDetection
    from ._models_py3 import ImageObjectDetectionBase
    from ._models_py3 import ImageSweepSettings
    from ._models_py3 import ImageVertical
    from ._models_py3 import IndexColumn
    from ._models_py3 import InferenceContainerProperties
    from ._models_py3 import InferencingServer
    from ._models_py3 import InstanceTypeSchema
    from ._models_py3 import InstanceTypeSchemaResources
    from ._models_py3 import JobBase
    from ._models_py3 import JobBaseProperties
    from ._models_py3 import JobBaseResourceArmPaginatedResult
    from ._models_py3 import JobInput
    from ._models_py3 import JobLimits
    from ._models_py3 import JobOutput
    from ._models_py3 import JobResourceConfiguration
    from ._models_py3 import JobScheduleAction
    from ._models_py3 import JobService
    from ._models_py3 import KerberosCredentials
    from ._models_py3 import KerberosKeytabCredentials
    from ._models_py3 import KerberosKeytabSecrets
    from ._models_py3 import KerberosPasswordCredentials
    from ._models_py3 import KerberosPasswordSecrets
    from ._models_py3 import Kubernetes
    from ._models_py3 import KubernetesOnlineDeployment
    from ._models_py3 import KubernetesProperties
    from ._models_py3 import KubernetesSchema
    from ._models_py3 import LabelCategory
    from ._models_py3 import LabelClass
    from ._models_py3 import LabelingDataConfiguration
    from ._models_py3 import LabelingJob
    from ._models_py3 import LabelingJobImageProperties
    from ._models_py3 import LabelingJobInstructions
    from ._models_py3 import LabelingJobMediaProperties
    from ._models_py3 import LabelingJobProperties
    from ._models_py3 import LabelingJobResourceArmPaginatedResult
    from ._models_py3 import LabelingJobTextProperties
    from ._models_py3 import ListAmlUserFeatureResult
    from ._models_py3 import ListNotebookKeysResult
    from ._models_py3 import ListStorageAccountKeysResult
    from ._models_py3 import ListUsagesResult
    from ._models_py3 import ListWorkspaceKeysResult
    from ._models_py3 import ListWorkspaceQuotas
    from ._models_py3 import LiteralJobInput
    from ._models_py3 import MLAssistConfiguration
    from ._models_py3 import MLAssistConfigurationDisabled
    from ._models_py3 import MLAssistConfigurationEnabled
    from ._models_py3 import MLFlowModelJobInput
    from ._models_py3 import MLFlowModelJobOutput
    from ._models_py3 import MLTableData
    from ._models_py3 import MLTableJobInput
    from ._models_py3 import MLTableJobOutput
    from ._models_py3 import ManagedIdentity
    from ._models_py3 import ManagedIdentityAuthTypeWorkspaceConnectionProperties
    from ._models_py3 import ManagedOnlineDeployment
    from ._models_py3 import ManagedServiceIdentity
    from ._models_py3 import MaterializationComputeResource
    from ._models_py3 import MaterializationSettings
    from ._models_py3 import MedianStoppingPolicy
    from ._models_py3 import ModelConfiguration
    from ._models_py3 import ModelContainer
    from ._models_py3 import ModelContainerProperties
    from ._models_py3 import ModelContainerResourceArmPaginatedResult
    from ._models_py3 import ModelPackageInput
    from ._models_py3 import ModelVersion
    from ._models_py3 import ModelVersionProperties
    from ._models_py3 import ModelVersionResourceArmPaginatedResult
    from ._models_py3 import Mpi
    from ._models_py3 import NCrossValidations
    from ._models_py3 import NlpFixedParameters
    from ._models_py3 import NlpParameterSubspace
    from ._models_py3 import NlpSweepSettings
    from ._models_py3 import NlpVertical
    from ._models_py3 import NlpVerticalFeaturizationSettings
    from ._models_py3 import NlpVerticalLimitSettings
    from ._models_py3 import NodeStateCounts
    from ._models_py3 import Nodes
    from ._models_py3 import NoneAuthTypeWorkspaceConnectionProperties
    from ._models_py3 import NoneDatastoreCredentials
    from ._models_py3 import NotebookAccessTokenResult
    from ._models_py3 import NotebookPreparationError
    from ._models_py3 import NotebookResourceInfo
    from ._models_py3 import NotificationSetting
    from ._models_py3 import Objective
    from ._models_py3 import OnlineDeployment
    from ._models_py3 import OnlineDeploymentProperties
    from ._models_py3 import OnlineDeploymentTrackedResourceArmPaginatedResult
    from ._models_py3 import OnlineEndpoint
    from ._models_py3 import OnlineEndpointProperties
    from ._models_py3 import OnlineEndpointTrackedResourceArmPaginatedResult
    from ._models_py3 import OnlineInferenceConfiguration
    from ._models_py3 import OnlineRequestSettings
    from ._models_py3 import OnlineScaleSettings
    from ._models_py3 import OutputPathAssetReference
    from ._models_py3 import PATAuthTypeWorkspaceConnectionProperties
    from ._models_py3 import PackageInputPathBase
    from ._models_py3 import PackageInputPathId
    from ._models_py3 import PackageInputPathUrl
    from ._models_py3 import PackageInputPathVersion
    from ._models_py3 import PackageRequest
    from ._models_py3 import PackageResponse
    from ._models_py3 import PaginatedComputeResourcesList
    from ._models_py3 import PartialBatchDeployment
    from ._models_py3 import PartialBatchDeploymentPartialMinimalTrackedResourceWithProperties
    from ._models_py3 import PartialManagedServiceIdentity
    from ._models_py3 import PartialMinimalTrackedResource
    from ._models_py3 import PartialMinimalTrackedResourceWithIdentity
    from ._models_py3 import PartialMinimalTrackedResourceWithSku
    from ._models_py3 import PartialRegistryPartialTrackedResource
    from ._models_py3 import PartialSku
    from ._models_py3 import Password
    from ._models_py3 import PersonalComputeInstanceSettings
    from ._models_py3 import PipelineJob
    from ._models_py3 import PrivateEndpoint
    from ._models_py3 import PrivateEndpointConnection
    from ._models_py3 import PrivateEndpointConnectionListResult
    from ._models_py3 import PrivateLinkResource
    from ._models_py3 import PrivateLinkResourceListResult
    from ._models_py3 import PrivateLinkServiceConnectionState
    from ._models_py3 import ProbeSettings
    from ._models_py3 import ProgressMetrics
    from ._models_py3 import PyTorch
    from ._models_py3 import QueueSettings
    from ._models_py3 import QuotaBaseProperties
    from ._models_py3 import QuotaUpdateParameters
    from ._models_py3 import RandomSamplingAlgorithm
    from ._models_py3 import RecurrenceSchedule
    from ._models_py3 import RecurrenceTrigger
    from ._models_py3 import RegenerateEndpointKeysRequest
    from ._models_py3 import Registry
    from ._models_py3 import RegistryListCredentialsResult
    from ._models_py3 import RegistryProperties
    from ._models_py3 import RegistryRegionArmDetails
    from ._models_py3 import RegistryTrackedResourceArmPaginatedResult
    from ._models_py3 import Regression
    from ._models_py3 import RegressionTrainingSettings
    from ._models_py3 import Resource
    from ._models_py3 import ResourceBase
    from ._models_py3 import ResourceConfiguration
    from ._models_py3 import ResourceId
    from ._models_py3 import ResourceName
    from ._models_py3 import ResourceQuota
    from ._models_py3 import Route
    from ._models_py3 import SASAuthTypeWorkspaceConnectionProperties
    from ._models_py3 import SamplingAlgorithm
    from ._models_py3 import SasDatastoreCredentials
    from ._models_py3 import SasDatastoreSecrets
    from ._models_py3 import ScaleSettings
    from ._models_py3 import ScaleSettingsInformation
    from ._models_py3 import Schedule
    from ._models_py3 import ScheduleActionBase
    from ._models_py3 import ScheduleBase
    from ._models_py3 import ScheduleProperties
    from ._models_py3 import ScheduleResourceArmPaginatedResult
    from ._models_py3 import ScriptReference
    from ._models_py3 import ScriptsToExecute
    from ._models_py3 import Seasonality
    from ._models_py3 import ServiceManagedResourcesSettings
    from ._models_py3 import ServicePrincipalAuthTypeWorkspaceConnectionProperties
    from ._models_py3 import ServicePrincipalDatastoreCredentials
    from ._models_py3 import ServicePrincipalDatastoreSecrets
    from ._models_py3 import SetupScripts
    from ._models_py3 import SharedPrivateLinkResource
    from ._models_py3 import Sku
    from ._models_py3 import SkuCapacity
    from ._models_py3 import SkuResource
    from ._models_py3 import SkuResourceArmPaginatedResult
    from ._models_py3 import SkuSetting
    from ._models_py3 import SparkJob
    from ._models_py3 import SparkJobEntry
    from ._models_py3 import SparkJobPythonEntry
    from ._models_py3 import SparkJobScalaEntry
    from ._models_py3 import SparkResourceConfiguration
    from ._models_py3 import SslConfiguration
    from ._models_py3 import StackEnsembleSettings
    from ._models_py3 import StatusMessage
    from ._models_py3 import StorageAccountDetails
    from ._models_py3 import SweepJob
    from ._models_py3 import SweepJobLimits
    from ._models_py3 import SynapseSpark
    from ._models_py3 import SynapseSparkProperties
    from ._models_py3 import SystemCreatedAcrAccount
    from ._models_py3 import SystemCreatedStorageAccount
    from ._models_py3 import SystemData
    from ._models_py3 import SystemService
    from ._models_py3 import TableFixedParameters
    from ._models_py3 import TableParameterSubspace
    from ._models_py3 import TableSweepSettings
    from ._models_py3 import TableVertical
    from ._models_py3 import TableVerticalFeaturizationSettings
    from ._models_py3 import TableVerticalLimitSettings
    from ._models_py3 import TargetLags
    from ._models_py3 import TargetRollingWindowSize
    from ._models_py3 import TargetUtilizationScaleSettings
    from ._models_py3 import TensorFlow
    from ._models_py3 import TextClassification
    from ._models_py3 import TextClassificationMultilabel
    from ._models_py3 import TextNer
    from ._models_py3 import TmpfsOptions
    from ._models_py3 import TrackedResource
    from ._models_py3 import TrainingSettings
    from ._models_py3 import TrialComponent
    from ._models_py3 import TriggerBase
    from ._models_py3 import TritonInferencingServer
    from ._models_py3 import TritonModelJobInput
    from ._models_py3 import TritonModelJobOutput
    from ._models_py3 import TruncationSelectionPolicy
    from ._models_py3 import UpdateWorkspaceQuotas
    from ._models_py3 import UpdateWorkspaceQuotasResult
    from ._models_py3 import UriFileDataVersion
    from ._models_py3 import UriFileJobInput
    from ._models_py3 import UriFileJobOutput
    from ._models_py3 import UriFolderDataVersion
    from ._models_py3 import UriFolderJobInput
    from ._models_py3 import UriFolderJobOutput
    from ._models_py3 import Usage
    from ._models_py3 import UsageName
    from ._models_py3 import UserAccountCredentials
    from ._models_py3 import UserAssignedIdentity
    from ._models_py3 import UserCreatedAcrAccount
    from ._models_py3 import UserCreatedStorageAccount
    from ._models_py3 import UserIdentity
    from ._models_py3 import UsernamePasswordAuthTypeWorkspaceConnectionProperties
    from ._models_py3 import VirtualMachine
    from ._models_py3 import VirtualMachineImage
    from ._models_py3 import VirtualMachineSchema
    from ._models_py3 import VirtualMachineSchemaProperties
    from ._models_py3 import VirtualMachineSecrets
    from ._models_py3 import VirtualMachineSecretsSchema
    from ._models_py3 import VirtualMachineSize
    from ._models_py3 import VirtualMachineSizeListResult
    from ._models_py3 import VirtualMachineSshCredentials
    from ._models_py3 import VolumeDefinition
    from ._models_py3 import VolumeOptions
    from ._models_py3 import Workspace
    from ._models_py3 import WorkspaceConnectionAccessKey
    from ._models_py3 import WorkspaceConnectionManagedIdentity
    from ._models_py3 import WorkspaceConnectionPersonalAccessToken
    from ._models_py3 import WorkspaceConnectionPropertiesV2
    from ._models_py3 import WorkspaceConnectionPropertiesV2BasicResource
    from ._models_py3 import WorkspaceConnectionPropertiesV2BasicResourceArmPaginatedResult
    from ._models_py3 import WorkspaceConnectionServicePrincipal
    from ._models_py3 import WorkspaceConnectionSharedAccessSignature
    from ._models_py3 import WorkspaceConnectionUsernamePassword
    from ._models_py3 import WorkspaceListResult
    from ._models_py3 import WorkspaceUpdateParameters
except (SyntaxError, ImportError):
    from ._models import AKS  # type: ignore
    from ._models import AKSSchema  # type: ignore
    from ._models import AKSSchemaProperties  # type: ignore
    from ._models import AccessKeyAuthTypeWorkspaceConnectionProperties  # type: ignore
    from ._models import AccountKeyDatastoreCredentials  # type: ignore
    from ._models import AccountKeyDatastoreSecrets  # type: ignore
    from ._models import AcrDetails  # type: ignore
    from ._models import AksComputeSecrets  # type: ignore
    from ._models import AksComputeSecretsProperties  # type: ignore
    from ._models import AksNetworkingConfiguration  # type: ignore
    from ._models import AllNodes  # type: ignore
    from ._models import AmlCompute  # type: ignore
    from ._models import AmlComputeNodeInformation  # type: ignore
    from ._models import AmlComputeNodesInformation  # type: ignore
    from ._models import AmlComputeProperties  # type: ignore
    from ._models import AmlComputeSchema  # type: ignore
    from ._models import AmlOperation  # type: ignore
    from ._models import AmlOperationDisplay  # type: ignore
    from ._models import AmlOperationListResult  # type: ignore
    from ._models import AmlToken  # type: ignore
    from ._models import AmlUserFeature  # type: ignore
    from ._models import ArmResourceId  # type: ignore
    from ._models import AssetBase  # type: ignore
    from ._models import AssetContainer  # type: ignore
    from ._models import AssetJobInput  # type: ignore
    from ._models import AssetJobOutput  # type: ignore
    from ._models import AssetReferenceBase  # type: ignore
    from ._models import AssignedUser  # type: ignore
    from ._models import AutoForecastHorizon  # type: ignore
    from ._models import AutoMLJob  # type: ignore
    from ._models import AutoMLVertical  # type: ignore
    from ._models import AutoNCrossValidations  # type: ignore
    from ._models import AutoPauseProperties  # type: ignore
    from ._models import AutoScaleProperties  # type: ignore
    from ._models import AutoSeasonality  # type: ignore
    from ._models import AutoTargetLags  # type: ignore
    from ._models import AutoTargetRollingWindowSize  # type: ignore
    from ._models import AutologgerSettings  # type: ignore
    from ._models import AzureBlobDatastore  # type: ignore
    from ._models import AzureDataLakeGen1Datastore  # type: ignore
    from ._models import AzureDataLakeGen2Datastore  # type: ignore
    from ._models import AzureDatastore  # type: ignore
    from ._models import AzureFileDatastore  # type: ignore
    from ._models import AzureMLBatchInferencingServer  # type: ignore
    from ._models import AzureMLOnlineInferencingServer  # type: ignore
    from ._models import BanditPolicy  # type: ignore
    from ._models import BaseEnvironmentId  # type: ignore
    from ._models import BaseEnvironmentSource  # type: ignore
    from ._models import BatchDeployment  # type: ignore
    from ._models import BatchDeploymentProperties  # type: ignore
    from ._models import BatchDeploymentPropertiesAutoGenerated  # type: ignore
    from ._models import BatchDeploymentTrackedResourceArmPaginatedResult  # type: ignore
    from ._models import BatchEndpoint  # type: ignore
    from ._models import BatchEndpointDefaults  # type: ignore
    from ._models import BatchEndpointProperties  # type: ignore
    from ._models import BatchEndpointTrackedResourceArmPaginatedResult  # type: ignore
    from ._models import BatchPipelineComponentDeploymentProperties  # type: ignore
    from ._models import BatchRetrySettings  # type: ignore
    from ._models import BayesianSamplingAlgorithm  # type: ignore
    from ._models import BindOptions  # type: ignore
    from ._models import BuildContext  # type: ignore
    from ._models import CertificateDatastoreCredentials  # type: ignore
    from ._models import CertificateDatastoreSecrets  # type: ignore
    from ._models import Classification  # type: ignore
    from ._models import ClassificationTrainingSettings  # type: ignore
    from ._models import ClusterUpdateParameters  # type: ignore
    from ._models import CocoExportSummary  # type: ignore
    from ._models import CodeConfiguration  # type: ignore
    from ._models import CodeContainer  # type: ignore
    from ._models import CodeContainerProperties  # type: ignore
    from ._models import CodeContainerResourceArmPaginatedResult  # type: ignore
    from ._models import CodeVersion  # type: ignore
    from ._models import CodeVersionProperties  # type: ignore
    from ._models import CodeVersionResourceArmPaginatedResult  # type: ignore
    from ._models import ColumnTransformer  # type: ignore
    from ._models import CommandJob  # type: ignore
    from ._models import CommandJobLimits  # type: ignore
    from ._models import ComponentContainer  # type: ignore
    from ._models import ComponentContainerProperties  # type: ignore
    from ._models import ComponentContainerResourceArmPaginatedResult  # type: ignore
    from ._models import ComponentVersion  # type: ignore
    from ._models import ComponentVersionProperties  # type: ignore
    from ._models import ComponentVersionResourceArmPaginatedResult  # type: ignore
    from ._models import Compute  # type: ignore
    from ._models import ComputeInstance  # type: ignore
    from ._models import ComputeInstanceApplication  # type: ignore
    from ._models import ComputeInstanceAutologgerSettings  # type: ignore
    from ._models import ComputeInstanceConnectivityEndpoints  # type: ignore
    from ._models import ComputeInstanceContainer  # type: ignore
    from ._models import ComputeInstanceCreatedBy  # type: ignore
    from ._models import ComputeInstanceDataDisk  # type: ignore
    from ._models import ComputeInstanceDataMount  # type: ignore
    from ._models import ComputeInstanceEnvironmentInfo  # type: ignore
    from ._models import ComputeInstanceLastOperation  # type: ignore
    from ._models import ComputeInstanceProperties  # type: ignore
    from ._models import ComputeInstanceSchema  # type: ignore
    from ._models import ComputeInstanceSshSettings  # type: ignore
    from ._models import ComputeInstanceVersion  # type: ignore
    from ._models import ComputeResource  # type: ignore
    from ._models import ComputeResourceSchema  # type: ignore
    from ._models import ComputeSchedules  # type: ignore
    from ._models import ComputeSecrets  # type: ignore
    from ._models import ComputeStartStopSchedule  # type: ignore
    from ._models import ContainerResourceRequirements  # type: ignore
    from ._models import ContainerResourceSettings  # type: ignore
    from ._models import CosmosDbSettings  # type: ignore
    from ._models import CronTrigger  # type: ignore
    from ._models import CsvExportSummary  # type: ignore
    from ._models import CustomForecastHorizon  # type: ignore
    from ._models import CustomInferencingServer  # type: ignore
    from ._models import CustomModelJobInput  # type: ignore
    from ._models import CustomModelJobOutput  # type: ignore
    from ._models import CustomNCrossValidations  # type: ignore
    from ._models import CustomSeasonality  # type: ignore
    from ._models import CustomService  # type: ignore
    from ._models import CustomTargetLags  # type: ignore
    from ._models import CustomTargetRollingWindowSize  # type: ignore
    from ._models import DataContainer  # type: ignore
    from ._models import DataContainerProperties  # type: ignore
    from ._models import DataContainerResourceArmPaginatedResult  # type: ignore
    from ._models import DataFactory  # type: ignore
    from ._models import DataLakeAnalytics  # type: ignore
    from ._models import DataLakeAnalyticsSchema  # type: ignore
    from ._models import DataLakeAnalyticsSchemaProperties  # type: ignore
    from ._models import DataPathAssetReference  # type: ignore
    from ._models import DataVersionBase  # type: ignore
    from ._models import DataVersionBaseProperties  # type: ignore
    from ._models import DataVersionBaseResourceArmPaginatedResult  # type: ignore
    from ._models import Databricks  # type: ignore
    from ._models import DatabricksComputeSecrets  # type: ignore
    from ._models import DatabricksComputeSecretsProperties  # type: ignore
    from ._models import DatabricksProperties  # type: ignore
    from ._models import DatabricksSchema  # type: ignore
    from ._models import DatasetExportSummary  # type: ignore
    from ._models import Datastore  # type: ignore
    from ._models import DatastoreCredentials  # type: ignore
    from ._models import DatastoreProperties  # type: ignore
    from ._models import DatastoreResourceArmPaginatedResult  # type: ignore
    from ._models import DatastoreSecrets  # type: ignore
    from ._models import DefaultScaleSettings  # type: ignore
    from ._models import DeploymentLogs  # type: ignore
    from ._models import DeploymentLogsRequest  # type: ignore
    from ._models import DeploymentResourceConfiguration  # type: ignore
    from ._models import DiagnoseRequestProperties  # type: ignore
    from ._models import DiagnoseResponseResult  # type: ignore
    from ._models import DiagnoseResponseResultValue  # type: ignore
    from ._models import DiagnoseResult  # type: ignore
    from ._models import DiagnoseWorkspaceParameters  # type: ignore
    from ._models import DistributionConfiguration  # type: ignore
    from ._models import Docker  # type: ignore
    from ._models import EarlyTerminationPolicy  # type: ignore
    from ._models import EncryptionKeyVaultProperties  # type: ignore
    from ._models import EncryptionKeyVaultUpdateProperties  # type: ignore
    from ._models import EncryptionProperty  # type: ignore
    from ._models import EncryptionUpdateProperties  # type: ignore
    from ._models import Endpoint  # type: ignore
    from ._models import EndpointAuthKeys  # type: ignore
    from ._models import EndpointAuthToken  # type: ignore
    from ._models import EndpointDeploymentPropertiesBase  # type: ignore
    from ._models import EndpointPropertiesBase  # type: ignore
    from ._models import EndpointScheduleAction  # type: ignore
    from ._models import EnvironmentContainer  # type: ignore
    from ._models import EnvironmentContainerProperties  # type: ignore
    from ._models import EnvironmentContainerResourceArmPaginatedResult  # type: ignore
    from ._models import EnvironmentVariable  # type: ignore
    from ._models import EnvironmentVersion  # type: ignore
    from ._models import EnvironmentVersionProperties  # type: ignore
    from ._models import EnvironmentVersionResourceArmPaginatedResult  # type: ignore
    from ._models import ErrorAdditionalInfo  # type: ignore
    from ._models import ErrorDetail  # type: ignore
    from ._models import ErrorResponse  # type: ignore
    from ._models import EstimatedVMPrice  # type: ignore
    from ._models import EstimatedVMPrices  # type: ignore
    from ._models import ExportSummary  # type: ignore
    from ._models import ExternalFQDNResponse  # type: ignore
    from ._models import FQDNEndpoint  # type: ignore
    from ._models import FQDNEndpointDetail  # type: ignore
    from ._models import FQDNEndpoints  # type: ignore
    from ._models import FQDNEndpointsProperties  # type: ignore
    from ._models import Feature  # type: ignore
    from ._models import FeatureArmPaginatedResult  # type: ignore
    from ._models import FeatureWindow  # type: ignore
    from ._models import FeaturesetContainer  # type: ignore
    from ._models import FeaturesetContainerProperties  # type: ignore
    from ._models import FeaturesetContainerResourceArmPaginatedResult  # type: ignore
    from ._models import FeaturesetJob  # type: ignore
    from ._models import FeaturesetJobArmPaginatedResult  # type: ignore
    from ._models import FeaturesetSpecification  # type: ignore
    from ._models import FeaturesetVersion  # type: ignore
    from ._models import FeaturesetVersionBackfillRequest  # type: ignore
    from ._models import FeaturesetVersionBackfillResponse  # type: ignore
    from ._models import FeaturesetVersionProperties  # type: ignore
    from ._models import FeaturesetVersionResourceArmPaginatedResult  # type: ignore
    from ._models import FeaturestoreEntityContainer  # type: ignore
    from ._models import FeaturestoreEntityContainerProperties  # type: ignore
    from ._models import FeaturestoreEntityContainerResourceArmPaginatedResult  # type: ignore
    from ._models import FeaturestoreEntityVersion  # type: ignore
    from ._models import FeaturestoreEntityVersionProperties  # type: ignore
    from ._models import FeaturestoreEntityVersionResourceArmPaginatedResult  # type: ignore
    from ._models import FeaturizationSettings  # type: ignore
    from ._models import FlavorData  # type: ignore
    from ._models import ForecastHorizon  # type: ignore
    from ._models import Forecasting  # type: ignore
    from ._models import ForecastingSettings  # type: ignore
    from ._models import ForecastingTrainingSettings  # type: ignore
    from ._models import GetFeatureRequest  # type: ignore
    from ._models import GridSamplingAlgorithm  # type: ignore
    from ._models import HDInsight  # type: ignore
    from ._models import HDInsightProperties  # type: ignore
    from ._models import HDInsightSchema  # type: ignore
    from ._models import HdfsDatastore  # type: ignore
    from ._models import IdAssetReference  # type: ignore
    from ._models import IdentityConfiguration  # type: ignore
    from ._models import IdentityForCmk  # type: ignore
    from ._models import IdleShutdownSetting  # type: ignore
    from ._models import Image  # type: ignore
    from ._models import ImageClassification  # type: ignore
    from ._models import ImageClassificationBase  # type: ignore
    from ._models import ImageClassificationMultilabel  # type: ignore
    from ._models import ImageInstanceSegmentation  # type: ignore
    from ._models import ImageLimitSettings  # type: ignore
    from ._models import ImageMetadata  # type: ignore
    from ._models import ImageModelDistributionSettings  # type: ignore
    from ._models import ImageModelDistributionSettingsClassification  # type: ignore
    from ._models import ImageModelDistributionSettingsObjectDetection  # type: ignore
    from ._models import ImageModelSettings  # type: ignore
    from ._models import ImageModelSettingsClassification  # type: ignore
    from ._models import ImageModelSettingsObjectDetection  # type: ignore
    from ._models import ImageObjectDetection  # type: ignore
    from ._models import ImageObjectDetectionBase  # type: ignore
    from ._models import ImageSweepSettings  # type: ignore
    from ._models import ImageVertical  # type: ignore
    from ._models import IndexColumn  # type: ignore
    from ._models import InferenceContainerProperties  # type: ignore
    from ._models import InferencingServer  # type: ignore
    from ._models import InstanceTypeSchema  # type: ignore
    from ._models import InstanceTypeSchemaResources  # type: ignore
    from ._models import JobBase  # type: ignore
    from ._models import JobBaseProperties  # type: ignore
    from ._models import JobBaseResourceArmPaginatedResult  # type: ignore
    from ._models import JobInput  # type: ignore
    from ._models import JobLimits  # type: ignore
    from ._models import JobOutput  # type: ignore
    from ._models import JobResourceConfiguration  # type: ignore
    from ._models import JobScheduleAction  # type: ignore
    from ._models import JobService  # type: ignore
    from ._models import KerberosCredentials  # type: ignore
    from ._models import KerberosKeytabCredentials  # type: ignore
    from ._models import KerberosKeytabSecrets  # type: ignore
    from ._models import KerberosPasswordCredentials  # type: ignore
    from ._models import KerberosPasswordSecrets  # type: ignore
    from ._models import Kubernetes  # type: ignore
    from ._models import KubernetesOnlineDeployment  # type: ignore
    from ._models import KubernetesProperties  # type: ignore
    from ._models import KubernetesSchema  # type: ignore
    from ._models import LabelCategory  # type: ignore
    from ._models import LabelClass  # type: ignore
    from ._models import LabelingDataConfiguration  # type: ignore
    from ._models import LabelingJob  # type: ignore
    from ._models import LabelingJobImageProperties  # type: ignore
    from ._models import LabelingJobInstructions  # type: ignore
    from ._models import LabelingJobMediaProperties  # type: ignore
    from ._models import LabelingJobProperties  # type: ignore
    from ._models import LabelingJobResourceArmPaginatedResult  # type: ignore
    from ._models import LabelingJobTextProperties  # type: ignore
    from ._models import ListAmlUserFeatureResult  # type: ignore
    from ._models import ListNotebookKeysResult  # type: ignore
    from ._models import ListStorageAccountKeysResult  # type: ignore
    from ._models import ListUsagesResult  # type: ignore
    from ._models import ListWorkspaceKeysResult  # type: ignore
    from ._models import ListWorkspaceQuotas  # type: ignore
    from ._models import LiteralJobInput  # type: ignore
    from ._models import MLAssistConfiguration  # type: ignore
    from ._models import MLAssistConfigurationDisabled  # type: ignore
    from ._models import MLAssistConfigurationEnabled  # type: ignore
    from ._models import MLFlowModelJobInput  # type: ignore
    from ._models import MLFlowModelJobOutput  # type: ignore
    from ._models import MLTableData  # type: ignore
    from ._models import MLTableJobInput  # type: ignore
    from ._models import MLTableJobOutput  # type: ignore
    from ._models import ManagedIdentity  # type: ignore
    from ._models import ManagedIdentityAuthTypeWorkspaceConnectionProperties  # type: ignore
    from ._models import ManagedOnlineDeployment  # type: ignore
    from ._models import ManagedServiceIdentity  # type: ignore
    from ._models import MaterializationComputeResource  # type: ignore
    from ._models import MaterializationSettings  # type: ignore
    from ._models import MedianStoppingPolicy  # type: ignore
    from ._models import ModelConfiguration  # type: ignore
    from ._models import ModelContainer  # type: ignore
    from ._models import ModelContainerProperties  # type: ignore
    from ._models import ModelContainerResourceArmPaginatedResult  # type: ignore
    from ._models import ModelPackageInput  # type: ignore
    from ._models import ModelVersion  # type: ignore
    from ._models import ModelVersionProperties  # type: ignore
    from ._models import ModelVersionResourceArmPaginatedResult  # type: ignore
    from ._models import Mpi  # type: ignore
    from ._models import NCrossValidations  # type: ignore
    from ._models import NlpFixedParameters  # type: ignore
    from ._models import NlpParameterSubspace  # type: ignore
    from ._models import NlpSweepSettings  # type: ignore
    from ._models import NlpVertical  # type: ignore
    from ._models import NlpVerticalFeaturizationSettings  # type: ignore
    from ._models import NlpVerticalLimitSettings  # type: ignore
    from ._models import NodeStateCounts  # type: ignore
    from ._models import Nodes  # type: ignore
    from ._models import NoneAuthTypeWorkspaceConnectionProperties  # type: ignore
    from ._models import NoneDatastoreCredentials  # type: ignore
    from ._models import NotebookAccessTokenResult  # type: ignore
    from ._models import NotebookPreparationError  # type: ignore
    from ._models import NotebookResourceInfo  # type: ignore
    from ._models import NotificationSetting  # type: ignore
    from ._models import Objective  # type: ignore
    from ._models import OnlineDeployment  # type: ignore
    from ._models import OnlineDeploymentProperties  # type: ignore
    from ._models import OnlineDeploymentTrackedResourceArmPaginatedResult  # type: ignore
    from ._models import OnlineEndpoint  # type: ignore
    from ._models import OnlineEndpointProperties  # type: ignore
    from ._models import OnlineEndpointTrackedResourceArmPaginatedResult  # type: ignore
    from ._models import OnlineInferenceConfiguration  # type: ignore
    from ._models import OnlineRequestSettings  # type: ignore
    from ._models import OnlineScaleSettings  # type: ignore
    from ._models import OutputPathAssetReference  # type: ignore
    from ._models import PATAuthTypeWorkspaceConnectionProperties  # type: ignore
    from ._models import PackageInputPathBase  # type: ignore
    from ._models import PackageInputPathId  # type: ignore
    from ._models import PackageInputPathUrl  # type: ignore
    from ._models import PackageInputPathVersion  # type: ignore
    from ._models import PackageRequest  # type: ignore
    from ._models import PackageResponse  # type: ignore
    from ._models import PaginatedComputeResourcesList  # type: ignore
    from ._models import PartialBatchDeployment  # type: ignore
    from ._models import PartialBatchDeploymentPartialMinimalTrackedResourceWithProperties  # type: ignore
    from ._models import PartialManagedServiceIdentity  # type: ignore
    from ._models import PartialMinimalTrackedResource  # type: ignore
    from ._models import PartialMinimalTrackedResourceWithIdentity  # type: ignore
    from ._models import PartialMinimalTrackedResourceWithSku  # type: ignore
    from ._models import PartialRegistryPartialTrackedResource  # type: ignore
    from ._models import PartialSku  # type: ignore
    from ._models import Password  # type: ignore
    from ._models import PersonalComputeInstanceSettings  # type: ignore
    from ._models import PipelineJob  # type: ignore
    from ._models import PrivateEndpoint  # type: ignore
    from ._models import PrivateEndpointConnection  # type: ignore
    from ._models import PrivateEndpointConnectionListResult  # type: ignore
    from ._models import PrivateLinkResource  # type: ignore
    from ._models import PrivateLinkResourceListResult  # type: ignore
    from ._models import PrivateLinkServiceConnectionState  # type: ignore
    from ._models import ProbeSettings  # type: ignore
    from ._models import ProgressMetrics  # type: ignore
    from ._models import PyTorch  # type: ignore
    from ._models import QueueSettings  # type: ignore
    from ._models import QuotaBaseProperties  # type: ignore
    from ._models import QuotaUpdateParameters  # type: ignore
    from ._models import RandomSamplingAlgorithm  # type: ignore
    from ._models import RecurrenceSchedule  # type: ignore
    from ._models import RecurrenceTrigger  # type: ignore
    from ._models import RegenerateEndpointKeysRequest  # type: ignore
    from ._models import Registry  # type: ignore
    from ._models import RegistryListCredentialsResult  # type: ignore
    from ._models import RegistryProperties  # type: ignore
    from ._models import RegistryRegionArmDetails  # type: ignore
    from ._models import RegistryTrackedResourceArmPaginatedResult  # type: ignore
    from ._models import Regression  # type: ignore
    from ._models import RegressionTrainingSettings  # type: ignore
    from ._models import Resource  # type: ignore
    from ._models import ResourceBase  # type: ignore
    from ._models import ResourceConfiguration  # type: ignore
    from ._models import ResourceId  # type: ignore
    from ._models import ResourceName  # type: ignore
    from ._models import ResourceQuota  # type: ignore
    from ._models import Route  # type: ignore
    from ._models import SASAuthTypeWorkspaceConnectionProperties  # type: ignore
    from ._models import SamplingAlgorithm  # type: ignore
    from ._models import SasDatastoreCredentials  # type: ignore
    from ._models import SasDatastoreSecrets  # type: ignore
    from ._models import ScaleSettings  # type: ignore
    from ._models import ScaleSettingsInformation  # type: ignore
    from ._models import Schedule  # type: ignore
    from ._models import ScheduleActionBase  # type: ignore
    from ._models import ScheduleBase  # type: ignore
    from ._models import ScheduleProperties  # type: ignore
    from ._models import ScheduleResourceArmPaginatedResult  # type: ignore
    from ._models import ScriptReference  # type: ignore
    from ._models import ScriptsToExecute  # type: ignore
    from ._models import Seasonality  # type: ignore
    from ._models import ServiceManagedResourcesSettings  # type: ignore
    from ._models import ServicePrincipalAuthTypeWorkspaceConnectionProperties  # type: ignore
    from ._models import ServicePrincipalDatastoreCredentials  # type: ignore
    from ._models import ServicePrincipalDatastoreSecrets  # type: ignore
    from ._models import SetupScripts  # type: ignore
    from ._models import SharedPrivateLinkResource  # type: ignore
    from ._models import Sku  # type: ignore
    from ._models import SkuCapacity  # type: ignore
    from ._models import SkuResource  # type: ignore
    from ._models import SkuResourceArmPaginatedResult  # type: ignore
    from ._models import SkuSetting  # type: ignore
    from ._models import SparkJob  # type: ignore
    from ._models import SparkJobEntry  # type: ignore
    from ._models import SparkJobPythonEntry  # type: ignore
    from ._models import SparkJobScalaEntry  # type: ignore
    from ._models import SparkResourceConfiguration  # type: ignore
    from ._models import SslConfiguration  # type: ignore
    from ._models import StackEnsembleSettings  # type: ignore
    from ._models import StatusMessage  # type: ignore
    from ._models import StorageAccountDetails  # type: ignore
    from ._models import SweepJob  # type: ignore
    from ._models import SweepJobLimits  # type: ignore
    from ._models import SynapseSpark  # type: ignore
    from ._models import SynapseSparkProperties  # type: ignore
    from ._models import SystemCreatedAcrAccount  # type: ignore
    from ._models import SystemCreatedStorageAccount  # type: ignore
    from ._models import SystemData  # type: ignore
    from ._models import SystemService  # type: ignore
    from ._models import TableFixedParameters  # type: ignore
    from ._models import TableParameterSubspace  # type: ignore
    from ._models import TableSweepSettings  # type: ignore
    from ._models import TableVertical  # type: ignore
    from ._models import TableVerticalFeaturizationSettings  # type: ignore
    from ._models import TableVerticalLimitSettings  # type: ignore
    from ._models import TargetLags  # type: ignore
    from ._models import TargetRollingWindowSize  # type: ignore
    from ._models import TargetUtilizationScaleSettings  # type: ignore
    from ._models import TensorFlow  # type: ignore
    from ._models import TextClassification  # type: ignore
    from ._models import TextClassificationMultilabel  # type: ignore
    from ._models import TextNer  # type: ignore
    from ._models import TmpfsOptions  # type: ignore
    from ._models import TrackedResource  # type: ignore
    from ._models import TrainingSettings  # type: ignore
    from ._models import TrialComponent  # type: ignore
    from ._models import TriggerBase  # type: ignore
    from ._models import TritonInferencingServer  # type: ignore
    from ._models import TritonModelJobInput  # type: ignore
    from ._models import TritonModelJobOutput  # type: ignore
    from ._models import TruncationSelectionPolicy  # type: ignore
    from ._models import UpdateWorkspaceQuotas  # type: ignore
    from ._models import UpdateWorkspaceQuotasResult  # type: ignore
    from ._models import UriFileDataVersion  # type: ignore
    from ._models import UriFileJobInput  # type: ignore
    from ._models import UriFileJobOutput  # type: ignore
    from ._models import UriFolderDataVersion  # type: ignore
    from ._models import UriFolderJobInput  # type: ignore
    from ._models import UriFolderJobOutput  # type: ignore
    from ._models import Usage  # type: ignore
    from ._models import UsageName  # type: ignore
    from ._models import UserAccountCredentials  # type: ignore
    from ._models import UserAssignedIdentity  # type: ignore
    from ._models import UserCreatedAcrAccount  # type: ignore
    from ._models import UserCreatedStorageAccount  # type: ignore
    from ._models import UserIdentity  # type: ignore
    from ._models import UsernamePasswordAuthTypeWorkspaceConnectionProperties  # type: ignore
    from ._models import VirtualMachine  # type: ignore
    from ._models import VirtualMachineImage  # type: ignore
    from ._models import VirtualMachineSchema  # type: ignore
    from ._models import VirtualMachineSchemaProperties  # type: ignore
    from ._models import VirtualMachineSecrets  # type: ignore
    from ._models import VirtualMachineSecretsSchema  # type: ignore
    from ._models import VirtualMachineSize  # type: ignore
    from ._models import VirtualMachineSizeListResult  # type: ignore
    from ._models import VirtualMachineSshCredentials  # type: ignore
    from ._models import VolumeDefinition  # type: ignore
    from ._models import VolumeOptions  # type: ignore
    from ._models import Workspace  # type: ignore
    from ._models import WorkspaceConnectionAccessKey  # type: ignore
    from ._models import WorkspaceConnectionManagedIdentity  # type: ignore
    from ._models import WorkspaceConnectionPersonalAccessToken  # type: ignore
    from ._models import WorkspaceConnectionPropertiesV2  # type: ignore
    from ._models import WorkspaceConnectionPropertiesV2BasicResource  # type: ignore
    from ._models import WorkspaceConnectionPropertiesV2BasicResourceArmPaginatedResult  # type: ignore
    from ._models import WorkspaceConnectionServicePrincipal  # type: ignore
    from ._models import WorkspaceConnectionSharedAccessSignature  # type: ignore
    from ._models import WorkspaceConnectionUsernamePassword  # type: ignore
    from ._models import WorkspaceListResult  # type: ignore
    from ._models import WorkspaceUpdateParameters  # type: ignore

from ._azure_machine_learning_workspaces_enums import (
    AllocationState,
    ApplicationSharingPolicy,
    AssetProvisioningState,
    AutoRebuildSetting,
    Autosave,
    BaseEnvironmentSourceType,
    BatchDeploymentPropertyType,
    BatchLoggingLevel,
    BatchOutputAction,
    BillingCurrency,
    BlockedTransformers,
    Caching,
    ClassificationModels,
    ClassificationMultilabelPrimaryMetrics,
    ClassificationPrimaryMetrics,
    ClusterPurpose,
    ComputeInstanceAuthorizationType,
    ComputeInstanceState,
    ComputePowerAction,
    ComputeType,
    ConnectionAuthType,
    ConnectionCategory,
    ContainerType,
    CreatedByType,
    CredentialsType,
    DataType,
    DatastoreType,
    DeploymentProvisioningState,
    DiagnoseResultLevel,
    DistributionType,
    EarlyTerminationPolicyType,
    EgressPublicNetworkAccessType,
    EmailNotificationEnableType,
    EncryptionStatus,
    EndpointAuthMode,
    EndpointComputeType,
    EndpointProvisioningState,
    EnvironmentType,
    EnvironmentVariableType,
    ExportFormatType,
    FeatureDataType,
    FeatureLags,
    FeaturestoreJobType,
    FeaturizationMode,
    ForecastHorizonMode,
    ForecastingModels,
    ForecastingPrimaryMetrics,
    Goal,
    IdentityConfigurationType,
    ImageAnnotationType,
    ImageType,
    IncrementalDataRefresh,
    InferencingServerType,
    InputDeliveryMode,
    InputPathType,
    InstanceSegmentationPrimaryMetrics,
    JobInputType,
    JobLimitsType,
    JobOutputType,
    JobProvisioningState,
    JobStatus,
    JobTier,
    JobType,
    KeyType,
    LearningRateScheduler,
    ListViewType,
    LoadBalancerType,
    LogVerbosity,
    MLAssistConfigurationType,
    MLFlowAutologgerState,
    ManagedServiceIdentityType,
    MaterializationStoreType,
    MediaType,
    MlflowAutologger,
    ModelSize,
    MountAction,
    MountState,
    MultiSelect,
    NCrossValidationsMode,
    Network,
    NlpLearningRateScheduler,
    NodeState,
    NodesValueType,
    ObjectDetectionPrimaryMetrics,
    OperatingSystemType,
    OperationName,
    OperationStatus,
    OperationTrigger,
    OrderString,
    OsType,
    OutputDeliveryMode,
    PackageBuildState,
    PackageInputDeliveryMode,
    PackageInputType,
    PrivateEndpointConnectionProvisioningState,
    PrivateEndpointServiceConnectionStatus,
    Protocol,
    ProvisioningState,
    ProvisioningStatus,
    PublicNetworkAccess,
    PublicNetworkAccessType,
    QuotaUnit,
    RandomSamplingAlgorithmRule,
    RecurrenceFrequency,
    ReferenceType,
    RegressionModels,
    RegressionPrimaryMetrics,
    RemoteLoginPortPublicAccess,
    SamplingAlgorithmType,
    ScaleType,
    ScheduleActionType,
    ScheduleListViewType,
    ScheduleProvisioningState,
    ScheduleProvisioningStatus,
    ScheduleStatus,
    SeasonalityMode,
    SecretsType,
    ServiceDataAccessAuthIdentity,
    ShortSeriesHandlingConfiguration,
    SkuScaleType,
    SkuTier,
    SourceType,
    SparkJobEntryType,
    SshPublicAccess,
    SslConfigStatus,
    StackMetaLearnerType,
    Status,
    StatusMessageLevel,
    StochasticOptimizer,
    StorageAccountType,
    TargetAggregationFunction,
    TargetLagsMode,
    TargetRollingWindowSizeMode,
    TaskType,
    TextAnnotationType,
    TrainingMode,
    TriggerType,
    UnderlyingResourceAction,
    UnitOfMeasure,
    UsageUnit,
    UseStl,
    VMPriceOSType,
    VMTier,
    ValidationMetricType,
    ValueFormat,
    VmPriority,
    VolumeDefinitionType,
    WeekDay,
)

__all__ = [
    "AKS",
    "AKSSchema",
    "AKSSchemaProperties",
    "AccessKeyAuthTypeWorkspaceConnectionProperties",
    "AccountKeyDatastoreCredentials",
    "AccountKeyDatastoreSecrets",
    "AcrDetails",
    "AksComputeSecrets",
    "AksComputeSecretsProperties",
    "AksNetworkingConfiguration",
    "AllNodes",
    "AmlCompute",
    "AmlComputeNodeInformation",
    "AmlComputeNodesInformation",
    "AmlComputeProperties",
    "AmlComputeSchema",
    "AmlOperation",
    "AmlOperationDisplay",
    "AmlOperationListResult",
    "AmlToken",
    "AmlUserFeature",
    "ArmResourceId",
    "AssetBase",
    "AssetContainer",
    "AssetJobInput",
    "AssetJobOutput",
    "AssetReferenceBase",
    "AssignedUser",
    "AutoForecastHorizon",
    "AutoMLJob",
    "AutoMLVertical",
    "AutoNCrossValidations",
    "AutoPauseProperties",
    "AutoScaleProperties",
    "AutoSeasonality",
    "AutoTargetLags",
    "AutoTargetRollingWindowSize",
    "AutologgerSettings",
    "AzureBlobDatastore",
    "AzureDataLakeGen1Datastore",
    "AzureDataLakeGen2Datastore",
    "AzureDatastore",
    "AzureFileDatastore",
    "AzureMLBatchInferencingServer",
    "AzureMLOnlineInferencingServer",
    "BanditPolicy",
    "BaseEnvironmentId",
    "BaseEnvironmentSource",
    "BatchDeployment",
    "BatchDeploymentProperties",
    "BatchDeploymentPropertiesAutoGenerated",
    "BatchDeploymentTrackedResourceArmPaginatedResult",
    "BatchEndpoint",
    "BatchEndpointDefaults",
    "BatchEndpointProperties",
    "BatchEndpointTrackedResourceArmPaginatedResult",
    "BatchPipelineComponentDeploymentProperties",
    "BatchRetrySettings",
    "BayesianSamplingAlgorithm",
    "BindOptions",
    "BuildContext",
    "CertificateDatastoreCredentials",
    "CertificateDatastoreSecrets",
    "Classification",
    "ClassificationTrainingSettings",
    "ClusterUpdateParameters",
    "CocoExportSummary",
    "CodeConfiguration",
    "CodeContainer",
    "CodeContainerProperties",
    "CodeContainerResourceArmPaginatedResult",
    "CodeVersion",
    "CodeVersionProperties",
    "CodeVersionResourceArmPaginatedResult",
    "ColumnTransformer",
    "CommandJob",
    "CommandJobLimits",
    "ComponentContainer",
    "ComponentContainerProperties",
    "ComponentContainerResourceArmPaginatedResult",
    "ComponentVersion",
    "ComponentVersionProperties",
    "ComponentVersionResourceArmPaginatedResult",
    "Compute",
    "ComputeInstance",
    "ComputeInstanceApplication",
    "ComputeInstanceAutologgerSettings",
    "ComputeInstanceConnectivityEndpoints",
    "ComputeInstanceContainer",
    "ComputeInstanceCreatedBy",
    "ComputeInstanceDataDisk",
    "ComputeInstanceDataMount",
    "ComputeInstanceEnvironmentInfo",
    "ComputeInstanceLastOperation",
    "ComputeInstanceProperties",
    "ComputeInstanceSchema",
    "ComputeInstanceSshSettings",
    "ComputeInstanceVersion",
    "ComputeResource",
    "ComputeResourceSchema",
    "ComputeSchedules",
    "ComputeSecrets",
    "ComputeStartStopSchedule",
    "ContainerResourceRequirements",
    "ContainerResourceSettings",
    "CosmosDbSettings",
    "CronTrigger",
    "CsvExportSummary",
    "CustomForecastHorizon",
    "CustomInferencingServer",
    "CustomModelJobInput",
    "CustomModelJobOutput",
    "CustomNCrossValidations",
    "CustomSeasonality",
    "CustomService",
    "CustomTargetLags",
    "CustomTargetRollingWindowSize",
    "DataContainer",
    "DataContainerProperties",
    "DataContainerResourceArmPaginatedResult",
    "DataFactory",
    "DataLakeAnalytics",
    "DataLakeAnalyticsSchema",
    "DataLakeAnalyticsSchemaProperties",
    "DataPathAssetReference",
    "DataVersionBase",
    "DataVersionBaseProperties",
    "DataVersionBaseResourceArmPaginatedResult",
    "Databricks",
    "DatabricksComputeSecrets",
    "DatabricksComputeSecretsProperties",
    "DatabricksProperties",
    "DatabricksSchema",
    "DatasetExportSummary",
    "Datastore",
    "DatastoreCredentials",
    "DatastoreProperties",
    "DatastoreResourceArmPaginatedResult",
    "DatastoreSecrets",
    "DefaultScaleSettings",
    "DeploymentLogs",
    "DeploymentLogsRequest",
    "DeploymentResourceConfiguration",
    "DiagnoseRequestProperties",
    "DiagnoseResponseResult",
    "DiagnoseResponseResultValue",
    "DiagnoseResult",
    "DiagnoseWorkspaceParameters",
    "DistributionConfiguration",
    "Docker",
    "EarlyTerminationPolicy",
    "EncryptionKeyVaultProperties",
    "EncryptionKeyVaultUpdateProperties",
    "EncryptionProperty",
    "EncryptionUpdateProperties",
    "Endpoint",
    "EndpointAuthKeys",
    "EndpointAuthToken",
    "EndpointDeploymentPropertiesBase",
    "EndpointPropertiesBase",
    "EndpointScheduleAction",
    "EnvironmentContainer",
    "EnvironmentContainerProperties",
    "EnvironmentContainerResourceArmPaginatedResult",
    "EnvironmentVariable",
    "EnvironmentVersion",
    "EnvironmentVersionProperties",
    "EnvironmentVersionResourceArmPaginatedResult",
    "ErrorAdditionalInfo",
    "ErrorDetail",
    "ErrorResponse",
    "EstimatedVMPrice",
    "EstimatedVMPrices",
    "ExportSummary",
    "ExternalFQDNResponse",
    "FQDNEndpoint",
    "FQDNEndpointDetail",
    "FQDNEndpoints",
    "FQDNEndpointsProperties",
    "Feature",
    "FeatureArmPaginatedResult",
    "FeatureWindow",
    "FeaturesetContainer",
    "FeaturesetContainerProperties",
    "FeaturesetContainerResourceArmPaginatedResult",
    "FeaturesetJob",
    "FeaturesetJobArmPaginatedResult",
    "FeaturesetSpecification",
    "FeaturesetVersion",
    "FeaturesetVersionBackfillRequest",
    "FeaturesetVersionBackfillResponse",
    "FeaturesetVersionProperties",
    "FeaturesetVersionResourceArmPaginatedResult",
    "FeaturestoreEntityContainer",
    "FeaturestoreEntityContainerProperties",
    "FeaturestoreEntityContainerResourceArmPaginatedResult",
    "FeaturestoreEntityVersion",
    "FeaturestoreEntityVersionProperties",
    "FeaturestoreEntityVersionResourceArmPaginatedResult",
    "FeaturizationSettings",
    "FlavorData",
    "ForecastHorizon",
    "Forecasting",
    "ForecastingSettings",
    "ForecastingTrainingSettings",
    "GetFeatureRequest",
    "GridSamplingAlgorithm",
    "HDInsight",
    "HDInsightProperties",
    "HDInsightSchema",
    "HdfsDatastore",
    "IdAssetReference",
    "IdentityConfiguration",
    "IdentityForCmk",
    "IdleShutdownSetting",
    "Image",
    "ImageClassification",
    "ImageClassificationBase",
    "ImageClassificationMultilabel",
    "ImageInstanceSegmentation",
    "ImageLimitSettings",
    "ImageMetadata",
    "ImageModelDistributionSettings",
    "ImageModelDistributionSettingsClassification",
    "ImageModelDistributionSettingsObjectDetection",
    "ImageModelSettings",
    "ImageModelSettingsClassification",
    "ImageModelSettingsObjectDetection",
    "ImageObjectDetection",
    "ImageObjectDetectionBase",
    "ImageSweepSettings",
    "ImageVertical",
    "IndexColumn",
    "InferenceContainerProperties",
    "InferencingServer",
    "InstanceTypeSchema",
    "InstanceTypeSchemaResources",
    "JobBase",
    "JobBaseProperties",
    "JobBaseResourceArmPaginatedResult",
    "JobInput",
    "JobLimits",
    "JobOutput",
    "JobResourceConfiguration",
    "JobScheduleAction",
    "JobService",
    "KerberosCredentials",
    "KerberosKeytabCredentials",
    "KerberosKeytabSecrets",
    "KerberosPasswordCredentials",
    "KerberosPasswordSecrets",
    "Kubernetes",
    "KubernetesOnlineDeployment",
    "KubernetesProperties",
    "KubernetesSchema",
    "LabelCategory",
    "LabelClass",
    "LabelingDataConfiguration",
    "LabelingJob",
    "LabelingJobImageProperties",
    "LabelingJobInstructions",
    "LabelingJobMediaProperties",
    "LabelingJobProperties",
    "LabelingJobResourceArmPaginatedResult",
    "LabelingJobTextProperties",
    "ListAmlUserFeatureResult",
    "ListNotebookKeysResult",
    "ListStorageAccountKeysResult",
    "ListUsagesResult",
    "ListWorkspaceKeysResult",
    "ListWorkspaceQuotas",
    "LiteralJobInput",
    "MLAssistConfiguration",
    "MLAssistConfigurationDisabled",
    "MLAssistConfigurationEnabled",
    "MLFlowModelJobInput",
    "MLFlowModelJobOutput",
    "MLTableData",
    "MLTableJobInput",
    "MLTableJobOutput",
    "ManagedIdentity",
    "ManagedIdentityAuthTypeWorkspaceConnectionProperties",
    "ManagedOnlineDeployment",
    "ManagedServiceIdentity",
    "MaterializationComputeResource",
    "MaterializationSettings",
    "MedianStoppingPolicy",
    "ModelConfiguration",
    "ModelContainer",
    "ModelContainerProperties",
    "ModelContainerResourceArmPaginatedResult",
    "ModelPackageInput",
    "ModelVersion",
    "ModelVersionProperties",
    "ModelVersionResourceArmPaginatedResult",
    "Mpi",
    "NCrossValidations",
    "NlpFixedParameters",
    "NlpParameterSubspace",
    "NlpSweepSettings",
    "NlpVertical",
    "NlpVerticalFeaturizationSettings",
    "NlpVerticalLimitSettings",
    "NodeStateCounts",
    "Nodes",
    "NoneAuthTypeWorkspaceConnectionProperties",
    "NoneDatastoreCredentials",
    "NotebookAccessTokenResult",
    "NotebookPreparationError",
    "NotebookResourceInfo",
    "NotificationSetting",
    "Objective",
    "OnlineDeployment",
    "OnlineDeploymentProperties",
    "OnlineDeploymentTrackedResourceArmPaginatedResult",
    "OnlineEndpoint",
    "OnlineEndpointProperties",
    "OnlineEndpointTrackedResourceArmPaginatedResult",
    "OnlineInferenceConfiguration",
    "OnlineRequestSettings",
    "OnlineScaleSettings",
    "OutputPathAssetReference",
    "PATAuthTypeWorkspaceConnectionProperties",
    "PackageInputPathBase",
    "PackageInputPathId",
    "PackageInputPathUrl",
    "PackageInputPathVersion",
    "PackageRequest",
    "PackageResponse",
    "PaginatedComputeResourcesList",
    "PartialBatchDeployment",
    "PartialBatchDeploymentPartialMinimalTrackedResourceWithProperties",
    "PartialManagedServiceIdentity",
    "PartialMinimalTrackedResource",
    "PartialMinimalTrackedResourceWithIdentity",
    "PartialMinimalTrackedResourceWithSku",
    "PartialRegistryPartialTrackedResource",
    "PartialSku",
    "Password",
    "PersonalComputeInstanceSettings",
    "PipelineJob",
    "PrivateEndpoint",
    "PrivateEndpointConnection",
    "PrivateEndpointConnectionListResult",
    "PrivateLinkResource",
    "PrivateLinkResourceListResult",
    "PrivateLinkServiceConnectionState",
    "ProbeSettings",
    "ProgressMetrics",
    "PyTorch",
    "QueueSettings",
    "QuotaBaseProperties",
    "QuotaUpdateParameters",
    "RandomSamplingAlgorithm",
    "RecurrenceSchedule",
    "RecurrenceTrigger",
    "RegenerateEndpointKeysRequest",
    "Registry",
    "RegistryListCredentialsResult",
    "RegistryProperties",
    "RegistryRegionArmDetails",
    "RegistryTrackedResourceArmPaginatedResult",
    "Regression",
    "RegressionTrainingSettings",
    "Resource",
    "ResourceBase",
    "ResourceConfiguration",
    "ResourceId",
    "ResourceName",
    "ResourceQuota",
    "Route",
    "SASAuthTypeWorkspaceConnectionProperties",
    "SamplingAlgorithm",
    "SasDatastoreCredentials",
    "SasDatastoreSecrets",
    "ScaleSettings",
    "ScaleSettingsInformation",
    "Schedule",
    "ScheduleActionBase",
    "ScheduleBase",
    "ScheduleProperties",
    "ScheduleResourceArmPaginatedResult",
    "ScriptReference",
    "ScriptsToExecute",
    "Seasonality",
    "ServiceManagedResourcesSettings",
    "ServicePrincipalAuthTypeWorkspaceConnectionProperties",
    "ServicePrincipalDatastoreCredentials",
    "ServicePrincipalDatastoreSecrets",
    "SetupScripts",
    "SharedPrivateLinkResource",
    "Sku",
    "SkuCapacity",
    "SkuResource",
    "SkuResourceArmPaginatedResult",
    "SkuSetting",
    "SparkJob",
    "SparkJobEntry",
    "SparkJobPythonEntry",
    "SparkJobScalaEntry",
    "SparkResourceConfiguration",
    "SslConfiguration",
    "StackEnsembleSettings",
    "StatusMessage",
    "StorageAccountDetails",
    "SweepJob",
    "SweepJobLimits",
    "SynapseSpark",
    "SynapseSparkProperties",
    "SystemCreatedAcrAccount",
    "SystemCreatedStorageAccount",
    "SystemData",
    "SystemService",
    "TableFixedParameters",
    "TableParameterSubspace",
    "TableSweepSettings",
    "TableVertical",
    "TableVerticalFeaturizationSettings",
    "TableVerticalLimitSettings",
    "TargetLags",
    "TargetRollingWindowSize",
    "TargetUtilizationScaleSettings",
    "TensorFlow",
    "TextClassification",
    "TextClassificationMultilabel",
    "TextNer",
    "TmpfsOptions",
    "TrackedResource",
    "TrainingSettings",
    "TrialComponent",
    "TriggerBase",
    "TritonInferencingServer",
    "TritonModelJobInput",
    "TritonModelJobOutput",
    "TruncationSelectionPolicy",
    "UpdateWorkspaceQuotas",
    "UpdateWorkspaceQuotasResult",
    "UriFileDataVersion",
    "UriFileJobInput",
    "UriFileJobOutput",
    "UriFolderDataVersion",
    "UriFolderJobInput",
    "UriFolderJobOutput",
    "Usage",
    "UsageName",
    "UserAccountCredentials",
    "UserAssignedIdentity",
    "UserCreatedAcrAccount",
    "UserCreatedStorageAccount",
    "UserIdentity",
    "UsernamePasswordAuthTypeWorkspaceConnectionProperties",
    "VirtualMachine",
    "VirtualMachineImage",
    "VirtualMachineSchema",
    "VirtualMachineSchemaProperties",
    "VirtualMachineSecrets",
    "VirtualMachineSecretsSchema",
    "VirtualMachineSize",
    "VirtualMachineSizeListResult",
    "VirtualMachineSshCredentials",
    "VolumeDefinition",
    "VolumeOptions",
    "Workspace",
    "WorkspaceConnectionAccessKey",
    "WorkspaceConnectionManagedIdentity",
    "WorkspaceConnectionPersonalAccessToken",
    "WorkspaceConnectionPropertiesV2",
    "WorkspaceConnectionPropertiesV2BasicResource",
    "WorkspaceConnectionPropertiesV2BasicResourceArmPaginatedResult",
    "WorkspaceConnectionServicePrincipal",
    "WorkspaceConnectionSharedAccessSignature",
    "WorkspaceConnectionUsernamePassword",
    "WorkspaceListResult",
    "WorkspaceUpdateParameters",
    "AllocationState",
    "ApplicationSharingPolicy",
    "AssetProvisioningState",
    "AutoRebuildSetting",
    "Autosave",
    "BaseEnvironmentSourceType",
    "BatchDeploymentPropertyType",
    "BatchLoggingLevel",
    "BatchOutputAction",
    "BillingCurrency",
    "BlockedTransformers",
    "Caching",
    "ClassificationModels",
    "ClassificationMultilabelPrimaryMetrics",
    "ClassificationPrimaryMetrics",
    "ClusterPurpose",
    "ComputeInstanceAuthorizationType",
    "ComputeInstanceState",
    "ComputePowerAction",
    "ComputeType",
    "ConnectionAuthType",
    "ConnectionCategory",
    "ContainerType",
    "CreatedByType",
    "CredentialsType",
    "DataType",
    "DatastoreType",
    "DeploymentProvisioningState",
    "DiagnoseResultLevel",
    "DistributionType",
    "EarlyTerminationPolicyType",
    "EgressPublicNetworkAccessType",
    "EmailNotificationEnableType",
    "EncryptionStatus",
    "EndpointAuthMode",
    "EndpointComputeType",
    "EndpointProvisioningState",
    "EnvironmentType",
    "EnvironmentVariableType",
    "ExportFormatType",
    "FeatureDataType",
    "FeatureLags",
    "FeaturestoreJobType",
    "FeaturizationMode",
    "ForecastHorizonMode",
    "ForecastingModels",
    "ForecastingPrimaryMetrics",
    "Goal",
    "IdentityConfigurationType",
    "ImageAnnotationType",
    "ImageType",
    "IncrementalDataRefresh",
    "InferencingServerType",
    "InputDeliveryMode",
    "InputPathType",
    "InstanceSegmentationPrimaryMetrics",
    "JobInputType",
    "JobLimitsType",
    "JobOutputType",
    "JobProvisioningState",
    "JobStatus",
    "JobTier",
    "JobType",
    "KeyType",
    "LearningRateScheduler",
    "ListViewType",
    "LoadBalancerType",
    "LogVerbosity",
    "MLAssistConfigurationType",
    "MLFlowAutologgerState",
    "ManagedServiceIdentityType",
    "MaterializationStoreType",
    "MediaType",
    "MlflowAutologger",
    "ModelSize",
    "MountAction",
    "MountState",
    "MultiSelect",
    "NCrossValidationsMode",
    "Network",
    "NlpLearningRateScheduler",
    "NodeState",
    "NodesValueType",
    "ObjectDetectionPrimaryMetrics",
    "OperatingSystemType",
    "OperationName",
    "OperationStatus",
    "OperationTrigger",
    "OrderString",
    "OsType",
    "OutputDeliveryMode",
    "PackageBuildState",
    "PackageInputDeliveryMode",
    "PackageInputType",
    "PrivateEndpointConnectionProvisioningState",
    "PrivateEndpointServiceConnectionStatus",
    "Protocol",
    "ProvisioningState",
    "ProvisioningStatus",
    "PublicNetworkAccess",
    "PublicNetworkAccessType",
    "QuotaUnit",
    "RandomSamplingAlgorithmRule",
    "RecurrenceFrequency",
    "ReferenceType",
    "RegressionModels",
    "RegressionPrimaryMetrics",
    "RemoteLoginPortPublicAccess",
    "SamplingAlgorithmType",
    "ScaleType",
    "ScheduleActionType",
    "ScheduleListViewType",
    "ScheduleProvisioningState",
    "ScheduleProvisioningStatus",
    "ScheduleStatus",
    "SeasonalityMode",
    "SecretsType",
    "ServiceDataAccessAuthIdentity",
    "ShortSeriesHandlingConfiguration",
    "SkuScaleType",
    "SkuTier",
    "SourceType",
    "SparkJobEntryType",
    "SshPublicAccess",
    "SslConfigStatus",
    "StackMetaLearnerType",
    "Status",
    "StatusMessageLevel",
    "StochasticOptimizer",
    "StorageAccountType",
    "TargetAggregationFunction",
    "TargetLagsMode",
    "TargetRollingWindowSizeMode",
    "TaskType",
    "TextAnnotationType",
    "TrainingMode",
    "TriggerType",
    "UnderlyingResourceAction",
    "UnitOfMeasure",
    "UsageUnit",
    "UseStl",
    "VMPriceOSType",
    "VMTier",
    "ValidationMetricType",
    "ValueFormat",
    "VmPriority",
    "VolumeDefinitionType",
    "WeekDay",
]



from pathlib import Path

from azure.ai.ml import Input, MLClient, dsl, load_component, load_data
from azure.ai.ml.entities import Data, PipelineJob
from azure.core.exceptions import ResourceNotFoundError

parent_dir = str(Path(__file__).parent)


def generate_dsl_pipeline(client: MLClient) -> PipelineJob:
    # 1. Load component funcs
    basic_func = load_component(source=parent_dir + "/component.yml")

    try:
        _ = client.data.get(name="sampledata1235", version="2")
    except ResourceNotFoundError:
        # Create the data version if not exits
        data = load_data(source=parent_dir + "/data.yml")
        _ = client.data.create_or_update(data)

    data_input = Input(type="uri_file", path="azureml:sampledata1235:2")
    mltable_input = Input(type="mltable", path="azureml:sampledata1235:2", mode="eval_mount")

    # 2. Construct pipeline
    @dsl.pipeline(
        description="Example of using data from a Dataset as pipeline input",
    )
    def dataset_input(
        pipeline_sample_input_data,
        pipeline_sample_input_string,
    ):
        hello_python_world_job = basic_func(
            sample_input_data=pipeline_sample_input_data,
            sample_input_string=pipeline_sample_input_string,
        )
        hello_python_world_job.compute = "cpu-cluster"

        mltable_job = basic_func(
            sample_input_data=mltable_input,
            sample_input_string=pipeline_sample_input_string,
        )
        mltable_job.compute = "cpu-cluster"

        return {
            "pipeline_sample_output_data": hello_python_world_job.outputs.sample_output_data,
        }

    pipeline = dataset_input(data_input, "Hello_Pipeline_World")
    pipeline.outputs.pipeline_sample_output_data.mode = "upload"
    return pipeline



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from typing import Optional
from azure.ai.ml.entities._mixins import RestTranslatableMixin
from azure.ai.ml._restclient.v2024_01_01_preview.models import (
    AzureOpenAiHyperParameters as RestAzureOpenAiHyperParameters,
)
from azure.ai.ml._utils._experimental import experimental


@experimental
class AzureOpenAIHyperparameters(RestTranslatableMixin):
    """Hyperparameters for Azure OpenAI model finetuning."""

    def __init__(
        self,
        *,
        batch_size: Optional[int] = None,
        learning_rate_multiplier: Optional[float] = None,
        n_epochs: Optional[int] = None,
    ):
        """Initialize AzureOpenAIHyperparameters.

        param batch_size: Number of examples in each batch.
                          A larger batch size means that model parameters are updated less
                          frequently, but with lower variance. Defaults to None.
        type batch_size: int
        param learning_rate_multiplier: Scaling factor for the learning rate.
                                        A smaller learning rate may be useful to avoid overfitting.
        type learning_rate_multiplier: float
        param n_epochs: The number of epochs to train the model for.
                        An epoch refers to one full cycle through the training dataset.
        type n_epochs: int
        """
        self._batch_size = batch_size
        self._learning_rate_multiplier = learning_rate_multiplier
        self._n_epochs = n_epochs
        # Not exposed in the public API, so need to check how to handle this
        # self._additional_properties = kwargs

    @property
    def batch_size(self) -> Optional[int]:
        """Get the batch size for training."""
        return self._batch_size

    @batch_size.setter
    def batch_size(self, value: Optional[int]) -> None:
        """Set the batch size for training.
        :param value: The batch size for training.
        :type value: int
        """
        self._batch_size = value

    @property
    def learning_rate_multiplier(self) -> Optional[float]:
        """Get the learning rate multiplier.
        :return: The learning rate multiplier.
        :rtype: float
        """
        return self._learning_rate_multiplier

    @learning_rate_multiplier.setter
    def learning_rate_multiplier(self, value: Optional[float]) -> None:
        """Set the learning rate multiplier.
        :param value: The learning rate multiplier.
        :type value: float
        """
        self._learning_rate_multiplier = value

    @property
    def n_epochs(self) -> Optional[int]:
        """Get the number of epochs.
        :return: The number of epochs.
        :rtype: int
        """
        return self._n_epochs

    @n_epochs.setter
    def n_epochs(self, value: Optional[int]) -> None:
        """Set the number of epochs.
        :param value: The number of epochs.
        :type value: int
        """
        self._n_epochs = value

    # Not exposed in the public API, so need to check how to handle this
    # @property
    # def additional_properties(self) -> dict:
    #    """Get additional properties."""
    #    return self._additional_properties

    # @additional_properties.setter
    # def additional_properties(self, value: dict) -> None:
    #    """Set additional properties."""
    #    self._additional_properties = value

    def _to_rest_object(self) -> RestAzureOpenAiHyperParameters:
        return RestAzureOpenAiHyperParameters(
            batch_size=self._batch_size,
            learning_rate_multiplier=self._learning_rate_multiplier,
            n_epochs=self._n_epochs,
        )

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, AzureOpenAIHyperparameters):
            return NotImplemented
        return (
            self._batch_size == other._batch_size
            and self._learning_rate_multiplier == other._learning_rate_multiplier
            and self._n_epochs == other._n_epochs
        )

    def __ne__(self, other: object) -> bool:
        return not self.__eq__(other)

    @classmethod
    def _from_rest_object(cls, obj: RestAzureOpenAiHyperParameters) -> "AzureOpenAIHyperparameters":
        aoai_hyperparameters = cls(
            batch_size=obj.batch_size,
            learning_rate_multiplier=obj.learning_rate_multiplier,
            n_epochs=obj.n_epochs,
        )
        return aoai_hyperparameters



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ----------------------------------------------------------


from typing import Optional

from azure.ai.ml._exception_helper import log_and_raise_error
from azure.ai.ml._restclient.v2023_04_01_preview.models import ModelConfiguration as RestModelConfiguration
from azure.ai.ml._utils._experimental import experimental
from azure.ai.ml.exceptions import ErrorCategory, ErrorTarget, ValidationErrorType, ValidationException


@experimental
class ModelConfiguration:
    """ModelConfiguration.

    :param mode: The mode of the model. Possible values include: "Copy", "Download".
    :type mode: str
    :param mount_path: The mount path of the model.
    :type mount_path: str

    .. admonition:: Example:

        .. literalinclude:: ../samples/ml_samples_misc.py
            :start-after: [START model_configuration_entity_create]
            :end-before: [END model_configuration_entity_create]
            :language: python
            :dedent: 8
            :caption: Creating a Model Configuration object.
    """

    def __init__(self, *, mode: Optional[str] = None, mount_path: Optional[str] = None):
        self.mode = mode
        self.mount_path = mount_path

    @classmethod
    def _from_rest_object(cls, rest_obj: RestModelConfiguration) -> "ModelConfiguration":
        return ModelConfiguration(mode=rest_obj.mode, mount_path=rest_obj.mount_path)

    def _to_rest_object(self) -> RestModelConfiguration:
        self._validate()
        return RestModelConfiguration(mode=self.mode, mount_path=self.mount_path)

    def _validate(self) -> None:
        if self.mode is not None and self.mode.lower() not in ["copy", "download"]:
            msg = "Mode must be either 'Copy' or 'Download'"
            err = ValidationException(
                message=msg,
                target=ErrorTarget.MODEL,
                no_personal_data_message=msg,
                error_category=ErrorCategory.USER_ERROR,
                error_type=ValidationErrorType.INVALID_VALUE,
            )
            log_and_raise_error(err)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

__path__ = __import__("pkgutil").extend_path(__path__, __name__)  # type: ignore

from .command_component import AnonymousCommandComponentSchema, CommandComponentSchema, ComponentFileRefField
from .component import ComponentSchema, ComponentYamlRefField
from .data_transfer_component import (
    AnonymousDataTransferCopyComponentSchema,
    AnonymousDataTransferExportComponentSchema,
    AnonymousDataTransferImportComponentSchema,
    DataTransferCopyComponentFileRefField,
    DataTransferCopyComponentSchema,
    DataTransferExportComponentFileRefField,
    DataTransferExportComponentSchema,
    DataTransferImportComponentFileRefField,
    DataTransferImportComponentSchema,
)
from .import_component import AnonymousImportComponentSchema, ImportComponentFileRefField, ImportComponentSchema
from .parallel_component import AnonymousParallelComponentSchema, ParallelComponentFileRefField, ParallelComponentSchema
from .spark_component import AnonymousSparkComponentSchema, SparkComponentFileRefField, SparkComponentSchema

__all__ = [
    "ComponentSchema",
    "CommandComponentSchema",
    "AnonymousCommandComponentSchema",
    "ComponentFileRefField",
    "ParallelComponentSchema",
    "AnonymousParallelComponentSchema",
    "ParallelComponentFileRefField",
    "ImportComponentSchema",
    "AnonymousImportComponentSchema",
    "ImportComponentFileRefField",
    "AnonymousSparkComponentSchema",
    "SparkComponentFileRefField",
    "SparkComponentSchema",
    "AnonymousDataTransferCopyComponentSchema",
    "DataTransferCopyComponentFileRefField",
    "DataTransferCopyComponentSchema",
    "AnonymousDataTransferImportComponentSchema",
    "DataTransferImportComponentFileRefField",
    "DataTransferImportComponentSchema",
    "AnonymousDataTransferExportComponentSchema",
    "DataTransferExportComponentFileRefField",
    "DataTransferExportComponentSchema",
    "ComponentYamlRefField",
]



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=unused-argument

from marshmallow import fields, post_load

from azure.ai.ml._schema.core.fields import StringTransformedEnum
from azure.ai.ml._utils.utils import camel_to_snake
from azure.ai.ml.constants._common import OneLakeArtifactTypes

from azure.ai.ml._schema.core.schema import PatchedSchemaMeta


class OneLakeArtifactSchema(metaclass=PatchedSchemaMeta):
    type = StringTransformedEnum(
        allowed_values=OneLakeArtifactTypes.ONE_LAKE, casing_transform=camel_to_snake, required=True
    )
    name = fields.Str(required=True)

    @post_load
    def make(self, data, **kwargs):
        from azure.ai.ml.entities import OneLakeConnectionArtifact

        return OneLakeConnectionArtifact(**data)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

__path__ = __import__("pkgutil").extend_path(__path__, __name__)  # type: ignore

from .adls_gen1 import AzureDataLakeGen1Schema
from .azure_storage import AzureBlobSchema, AzureDataLakeGen2Schema, AzureFileSchema, AzureStorageSchema
from .credentials import (
    AccountKeySchema,
    BaseTenantCredentialSchema,
    CertificateSchema,
    NoneCredentialsSchema,
    SasTokenSchema,
    ServicePrincipalSchema,
)

__all__ = [
    "AccountKeySchema",
    "AzureBlobSchema",
    "AzureDataLakeGen1Schema",
    "AzureDataLakeGen2Schema",
    "AzureFileSchema",
    "AzureStorageSchema",
    "BaseTenantCredentialSchema",
    "CertificateSchema",
    "NoneCredentialsSchema",
    "SasTokenSchema",
    "ServicePrincipalSchema",
]



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

# pylint: disable=unused-argument

from marshmallow import fields, post_load

from azure.ai.ml._restclient.v2023_04_01_preview.models import (
    LearningRateScheduler,
    ModelSize,
    StochasticOptimizer,
    ValidationMetricType,
)
from azure.ai.ml._schema.core.fields import StringTransformedEnum
from azure.ai.ml._schema.core.schema import PatchedSchemaMeta
from azure.ai.ml._utils.utils import camel_to_snake


class ImageModelSettingsSchema(metaclass=PatchedSchemaMeta):
    ams_gradient = fields.Bool()
    advanced_settings = fields.Str()
    beta1 = fields.Float()
    beta2 = fields.Float()
    checkpoint_frequency = fields.Int()
    checkpoint_run_id = fields.Str()
    distributed = fields.Bool()
    early_stopping = fields.Bool()
    early_stopping_delay = fields.Int()
    early_stopping_patience = fields.Int()
    evaluation_frequency = fields.Int()
    enable_onnx_normalization = fields.Bool()
    gradient_accumulation_step = fields.Int()
    layers_to_freeze = fields.Int()
    learning_rate = fields.Float()
    learning_rate_scheduler = StringTransformedEnum(
        allowed_values=[o.value for o in LearningRateScheduler],
        casing_transform=camel_to_snake,
    )
    model_name = fields.Str()
    momentum = fields.Float()
    nesterov = fields.Bool()
    number_of_epochs = fields.Int()
    number_of_workers = fields.Int()
    optimizer = StringTransformedEnum(
        allowed_values=[o.value for o in StochasticOptimizer],
        casing_transform=camel_to_snake,
    )
    random_seed = fields.Int()
    step_lr_gamma = fields.Float()
    step_lr_step_size = fields.Int()
    training_batch_size = fields.Int()
    validation_batch_size = fields.Int()
    warmup_cosine_lr_cycles = fields.Float()
    warmup_cosine_lr_warmup_epochs = fields.Int()
    weight_decay = fields.Float()


class ImageModelSettingsClassificationSchema(ImageModelSettingsSchema):
    training_crop_size = fields.Int()
    validation_crop_size = fields.Int()
    validation_resize_size = fields.Int()
    weighted_loss = fields.Int()

    @post_load
    def make(self, data, **kwargs):
        from azure.ai.ml.entities._job.automl.image.image_model_settings import ImageModelSettingsClassification

        return ImageModelSettingsClassification(**data)


class ImageModelSettingsObjectDetectionSchema(ImageModelSettingsSchema):
    box_detections_per_image = fields.Int()
    box_score_threshold = fields.Float()
    image_size = fields.Int()
    max_size = fields.Int()
    min_size = fields.Int()
    model_size = StringTransformedEnum(allowed_values=[o.value for o in ModelSize], casing_transform=camel_to_snake)
    multi_scale = fields.Bool()
    nms_iou_threshold = fields.Float()
    tile_grid_size = fields.Str()
    tile_overlap_ratio = fields.Float()
    tile_predictions_nms_threshold = fields.Float()
    validation_iou_threshold = fields.Float()
    validation_metric_type = StringTransformedEnum(
        allowed_values=[o.value for o in ValidationMetricType],
        casing_transform=camel_to_snake,
    )
    log_training_metrics = fields.Str()
    log_validation_loss = fields.Str()

    @post_load
    def make(self, data, **kwargs):
        from azure.ai.ml.entities._job.automl.image.image_model_settings import ImageModelSettingsObjectDetection

        return ImageModelSettingsObjectDetection(**data)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from typing import Any, List, Optional

from azure.ai.ml._restclient.v2023_10_01.models import (
    FeaturesetVersionBackfillResponse as RestFeaturesetVersionBackfillResponse,
)
from azure.ai.ml.entities._mixins import RestTranslatableMixin


class FeatureSetBackfillMetadata(RestTranslatableMixin):
    """Feature Set Backfill Metadata

    :param job_ids: A list of IDs of the backfill jobs. Defaults to None.
    :type job_ids: Optional[List[str]]
    :param type: The type of the backfill job. Defaults to None.
    :type type: Optional[str]
    :param kwargs: A dictionary of additional configuration parameters.
    :type kwargs: dict
    """

    def __init__(
        self,
        *,
        job_ids: Optional[List[str]] = None,
        type: Optional[str] = None,  # pylint: disable=redefined-builtin
        # pylint: disable=unused-argument
        **kwargs: Any
    ) -> None:
        self.type = type if type else "BackfillMaterialization"
        self.job_ids = job_ids

    @classmethod
    def _from_rest_object(cls, obj: RestFeaturesetVersionBackfillResponse) -> Optional["FeatureSetBackfillMetadata"]:
        if not obj:
            return None
        return FeatureSetBackfillMetadata(job_ids=obj.job_ids)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import functools
from typing import TYPE_CHECKING
import warnings

from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, ResourceNotFoundError, map_error
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import HttpResponse
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.mgmt.core.exceptions import ARMErrorFormat
from msrest import Serializer

from .. import models as _models
from .._vendor import _convert_request, _format_url_section

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from typing import Any, Callable, Dict, Generic, Optional, TypeVar
    T = TypeVar('T')
    ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False
# fmt: off

def build_registry_management_non_workspace_request(
    registry_name,  # type: str
    **kwargs  # type: Any
):
    # type: (...) -> HttpRequest
    accept = "application/json, text/json"
    # Construct URL
    url = kwargs.pop("template_url", '/registrymanagement/v1.0/registries/{registryName}/discovery')
    path_format_arguments = {
        "registryName": _SERIALIZER.url("registry_name", registry_name, 'str'),
    }

    url = _format_url_section(url, **path_format_arguments)

    # Construct headers
    header_parameters = kwargs.pop("headers", {})  # type: Dict[str, Any]
    header_parameters['Accept'] = _SERIALIZER.header("accept", accept, 'str')

    return HttpRequest(
        method="GET",
        url=url,
        headers=header_parameters,
        **kwargs
    )

# fmt: on
class RegistryManagementNonWorkspaceOperations(object):
    """RegistryManagementNonWorkspaceOperations operations.

    You should not instantiate this class directly. Instead, you should create a Client instance that
    instantiates it for you and attaches it as an attribute.

    :ivar models: Alias to model classes used in this operation group.
    :type models: ~azure.mgmt.machinelearningservices.models
    :param client: Client for service requests.
    :param config: Configuration of service client.
    :param serializer: An object model serializer.
    :param deserializer: An object model deserializer.
    """

    models = _models

    def __init__(self, client, config, serializer, deserializer):
        self._client = client
        self._serialize = serializer
        self._deserialize = deserializer
        self._config = config

    @distributed_trace
    def registry_management_non_workspace(
        self,
        registry_name,  # type: str
        **kwargs  # type: Any
    ):
        # type: (...) -> "_models.RegistryDiscoveryDto"
        """registry_management_non_workspace.

        :param registry_name:
        :type registry_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: RegistryDiscoveryDto, or the result of cls(response)
        :rtype: ~azure.mgmt.machinelearningservices.models.RegistryDiscoveryDto
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.RegistryDiscoveryDto"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        
        request = build_registry_management_non_workspace_request(
            registry_name=registry_name,
            template_url=self.registry_management_non_workspace.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = self._client._pipeline.run(request, stream=False, **kwargs)
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = self._deserialize('RegistryDiscoveryDto', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    registry_management_non_workspace.metadata = {'url': '/registrymanagement/v1.0/registries/{registryName}/discovery'}  # type: ignore




from pathlib import Path

from azure.ai.ml import Output, dsl
from azure.ai.ml.data_transfer import import_data
from azure.ai.ml.constants._common import AssetTypes, SERVERLESS_COMPUTE
from azure.ai.ml.entities import PipelineJob


parent_dir = str(Path(__file__).parent)


def generate_dsl_pipeline_from_builder() -> PipelineJob:
    path_source_s3 = "test1/*"
    connection_target = "azureml:my-s3-connection"
    outputs = {"sink": Output(type=AssetTypes.URI_FOLDER)}
    source = {"type": "file_system", "connection": connection_target, "path": path_source_s3}

    @dsl.pipeline(description="submit a pipeline with data transfer import file system job")
    def data_transfer_import_file_system_pipeline_from_builder(path_source_s3, connection_target):
        from azure.ai.ml.data_transfer import FileSystem

        s3_blob_input = import_data(source=source, outputs=outputs)

        source_snowflake = FileSystem(path=path_source_s3, connection=connection_target)
        s3_blob = import_data(source=source_snowflake, outputs=outputs)

    pipeline = data_transfer_import_file_system_pipeline_from_builder(path_source_s3, connection_target)
    pipeline.settings.default_compute = SERVERLESS_COMPUTE
    return pipeline



# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from typing import Any, AsyncIterable, Callable, Dict, Optional, TypeVar, Union

from azure.core.async_paging import AsyncItemPaged, AsyncList
from azure.core.exceptions import ClientAuthenticationError, HttpResponseError, ResourceExistsError, ResourceNotFoundError, map_error
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import AsyncHttpResponse
from azure.core.polling import AsyncLROPoller, AsyncNoPolling, AsyncPollingMethod
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.mgmt.core.exceptions import ARMErrorFormat
from azure.mgmt.core.polling.async_arm_polling import AsyncARMPolling

from ... import models as _models
from ..._vendor import _convert_request
from ...operations._featureset_versions_operations import build_backfill_request_initial, build_create_or_update_request_initial, build_delete_request_initial, build_get_request, build_list_request
T = TypeVar('T')
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]

class FeaturesetVersionsOperations:
    """FeaturesetVersionsOperations async operations.

    You should not instantiate this class directly. Instead, you should create a Client instance that
    instantiates it for you and attaches it as an attribute.

    :ivar models: Alias to model classes used in this operation group.
    :type models: ~azure.mgmt.machinelearningservices.models
    :param client: Client for service requests.
    :param config: Configuration of service client.
    :param serializer: An object model serializer.
    :param deserializer: An object model deserializer.
    """

    models = _models

    def __init__(self, client, config, serializer, deserializer) -> None:
        self._client = client
        self._serialize = serializer
        self._deserialize = deserializer
        self._config = config

    @distributed_trace
    def list(
        self,
        resource_group_name: str,
        workspace_name: str,
        name: str,
        skip: Optional[str] = None,
        tags: Optional[str] = None,
        list_view_type: Optional[Union[str, "_models.ListViewType"]] = None,
        page_size: Optional[int] = 20,
        version_name: Optional[str] = None,
        version: Optional[str] = None,
        description: Optional[str] = None,
        created_by: Optional[str] = None,
        stage: Optional[str] = None,
        **kwargs: Any
    ) -> AsyncIterable["_models.FeaturesetVersionResourceArmPaginatedResult"]:
        """List versions.

        List versions.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace.
        :type workspace_name: str
        :param name: Featureset name. This is case-sensitive.
        :type name: str
        :param skip: Continuation token for pagination.
        :type skip: str
        :param tags: Comma-separated list of tag names (and optionally values). Example:
         tag1,tag2=value2.
        :type tags: str
        :param list_view_type: [ListViewType.ActiveOnly, ListViewType.ArchivedOnly,
         ListViewType.All]View type for including/excluding (for example) archived entities.
        :type list_view_type: str or ~azure.mgmt.machinelearningservices.models.ListViewType
        :param page_size: page size.
        :type page_size: int
        :param version_name: name for the featureset version.
        :type version_name: str
        :param version: featureset version.
        :type version: str
        :param description: description for the feature set version.
        :type description: str
        :param created_by: createdBy user name.
        :type created_by: str
        :param stage: Specifies the featurestore stage.
        :type stage: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either FeaturesetVersionResourceArmPaginatedResult or the
         result of cls(response)
        :rtype:
         ~azure.core.async_paging.AsyncItemPaged[~azure.mgmt.machinelearningservices.models.FeaturesetVersionResourceArmPaginatedResult]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        api_version = kwargs.pop('api_version', "2023-08-01-preview")  # type: str

        cls = kwargs.pop('cls', None)  # type: ClsType["_models.FeaturesetVersionResourceArmPaginatedResult"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))
        def prepare_request(next_link=None):
            if not next_link:
                
                request = build_list_request(
                    subscription_id=self._config.subscription_id,
                    resource_group_name=resource_group_name,
                    workspace_name=workspace_name,
                    name=name,
                    api_version=api_version,
                    skip=skip,
                    tags=tags,
                    list_view_type=list_view_type,
                    page_size=page_size,
                    version_name=version_name,
                    version=version,
                    description=description,
                    created_by=created_by,
                    stage=stage,
                    template_url=self.list.metadata['url'],
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)

            else:
                
                request = build_list_request(
                    subscription_id=self._config.subscription_id,
                    resource_group_name=resource_group_name,
                    workspace_name=workspace_name,
                    name=name,
                    api_version=api_version,
                    skip=skip,
                    tags=tags,
                    list_view_type=list_view_type,
                    page_size=page_size,
                    version_name=version_name,
                    version=version,
                    description=description,
                    created_by=created_by,
                    stage=stage,
                    template_url=next_link,
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)
                request.method = "GET"
            return request

        async def extract_data(pipeline_response):
            deserialized = self._deserialize("FeaturesetVersionResourceArmPaginatedResult", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            request = prepare_request(next_link)

            pipeline_response = await self._client._pipeline.run(  # pylint: disable=protected-access
                request,
                stream=False,
                **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response


        return AsyncItemPaged(
            get_next, extract_data
        )
    list.metadata = {'url': "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/featuresets/{name}/versions"}  # type: ignore

    async def _delete_initial(  # pylint: disable=inconsistent-return-statements
        self,
        resource_group_name: str,
        workspace_name: str,
        name: str,
        version: str,
        **kwargs: Any
    ) -> None:
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        api_version = kwargs.pop('api_version', "2023-08-01-preview")  # type: str

        
        request = build_delete_request_initial(
            subscription_id=self._config.subscription_id,
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            name=name,
            version=version,
            api_version=api_version,
            template_url=self._delete_initial.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200, 202, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        response_headers = {}
        if response.status_code == 202:
            response_headers['x-ms-async-operation-timeout']=self._deserialize('duration', response.headers.get('x-ms-async-operation-timeout'))
            response_headers['Location']=self._deserialize('str', response.headers.get('Location'))
            response_headers['Retry-After']=self._deserialize('int', response.headers.get('Retry-After'))
            

        if cls:
            return cls(pipeline_response, None, response_headers)

    _delete_initial.metadata = {'url': "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/featuresets/{name}/versions/{version}"}  # type: ignore


    @distributed_trace_async
    async def begin_delete(  # pylint: disable=inconsistent-return-statements
        self,
        resource_group_name: str,
        workspace_name: str,
        name: str,
        version: str,
        **kwargs: Any
    ) -> AsyncLROPoller[None]:
        """Delete version.

        Delete version.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace.
        :type workspace_name: str
        :param name: Container name. This is case-sensitive.
        :type name: str
        :param version: Version identifier. This is case-sensitive.
        :type version: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be AsyncARMPolling. Pass in False for
         this operation to not poll, or pass in your own initialized polling object for a personal
         polling strategy.
        :paramtype polling: bool or ~azure.core.polling.AsyncPollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of AsyncLROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.AsyncLROPoller[None]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        api_version = kwargs.pop('api_version', "2023-08-01-preview")  # type: str
        polling = kwargs.pop('polling', True)  # type: Union[bool, AsyncPollingMethod]
        cls = kwargs.pop('cls', None)  # type: ClsType[None]
        lro_delay = kwargs.pop(
            'polling_interval',
            self._config.polling_interval
        )
        cont_token = kwargs.pop('continuation_token', None)  # type: Optional[str]
        if cont_token is None:
            raw_result = await self._delete_initial(
                resource_group_name=resource_group_name,
                workspace_name=workspace_name,
                name=name,
                version=version,
                api_version=api_version,
                cls=lambda x,y,z: x,
                **kwargs
            )
        kwargs.pop('error_map', None)

        def get_long_running_output(pipeline_response):
            if cls:
                return cls(pipeline_response, None, {})


        if polling is True: polling_method = AsyncARMPolling(lro_delay, lro_options={'final-state-via': 'location'}, **kwargs)
        elif polling is False: polling_method = AsyncNoPolling()
        else: polling_method = polling
        if cont_token:
            return AsyncLROPoller.from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output
            )
        return AsyncLROPoller(self._client, raw_result, get_long_running_output, polling_method)

    begin_delete.metadata = {'url': "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/featuresets/{name}/versions/{version}"}  # type: ignore

    @distributed_trace_async
    async def get(
        self,
        resource_group_name: str,
        workspace_name: str,
        name: str,
        version: str,
        **kwargs: Any
    ) -> "_models.FeaturesetVersion":
        """Get version.

        Get version.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace.
        :type workspace_name: str
        :param name: Container name. This is case-sensitive.
        :type name: str
        :param version: Version identifier. This is case-sensitive.
        :type version: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: FeaturesetVersion, or the result of cls(response)
        :rtype: ~azure.mgmt.machinelearningservices.models.FeaturesetVersion
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.FeaturesetVersion"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        api_version = kwargs.pop('api_version', "2023-08-01-preview")  # type: str

        
        request = build_get_request(
            subscription_id=self._config.subscription_id,
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            name=name,
            version=version,
            api_version=api_version,
            template_url=self.get.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize('FeaturesetVersion', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get.metadata = {'url': "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/featuresets/{name}/versions/{version}"}  # type: ignore


    async def _create_or_update_initial(
        self,
        resource_group_name: str,
        workspace_name: str,
        name: str,
        version: str,
        body: "_models.FeaturesetVersion",
        **kwargs: Any
    ) -> "_models.FeaturesetVersion":
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.FeaturesetVersion"]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        api_version = kwargs.pop('api_version', "2023-08-01-preview")  # type: str
        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        _json = self._serialize.body(body, 'FeaturesetVersion')

        request = build_create_or_update_request_initial(
            subscription_id=self._config.subscription_id,
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            name=name,
            version=version,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            template_url=self._create_or_update_initial.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        response_headers = {}
        if response.status_code == 200:
            deserialized = self._deserialize('FeaturesetVersion', pipeline_response)

        if response.status_code == 201:
            response_headers['x-ms-async-operation-timeout']=self._deserialize('duration', response.headers.get('x-ms-async-operation-timeout'))
            response_headers['Azure-AsyncOperation']=self._deserialize('str', response.headers.get('Azure-AsyncOperation'))
            
            deserialized = self._deserialize('FeaturesetVersion', pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, response_headers)

        return deserialized

    _create_or_update_initial.metadata = {'url': "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/featuresets/{name}/versions/{version}"}  # type: ignore


    @distributed_trace_async
    async def begin_create_or_update(
        self,
        resource_group_name: str,
        workspace_name: str,
        name: str,
        version: str,
        body: "_models.FeaturesetVersion",
        **kwargs: Any
    ) -> AsyncLROPoller["_models.FeaturesetVersion"]:
        """Create or update version.

        Create or update version.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace.
        :type workspace_name: str
        :param name: Container name. This is case-sensitive.
        :type name: str
        :param version: Version identifier. This is case-sensitive.
        :type version: str
        :param body: Version entity to create or update.
        :type body: ~azure.mgmt.machinelearningservices.models.FeaturesetVersion
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be AsyncARMPolling. Pass in False for
         this operation to not poll, or pass in your own initialized polling object for a personal
         polling strategy.
        :paramtype polling: bool or ~azure.core.polling.AsyncPollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of AsyncLROPoller that returns either FeaturesetVersion or the result of
         cls(response)
        :rtype:
         ~azure.core.polling.AsyncLROPoller[~azure.mgmt.machinelearningservices.models.FeaturesetVersion]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        api_version = kwargs.pop('api_version', "2023-08-01-preview")  # type: str
        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]
        polling = kwargs.pop('polling', True)  # type: Union[bool, AsyncPollingMethod]
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.FeaturesetVersion"]
        lro_delay = kwargs.pop(
            'polling_interval',
            self._config.polling_interval
        )
        cont_token = kwargs.pop('continuation_token', None)  # type: Optional[str]
        if cont_token is None:
            raw_result = await self._create_or_update_initial(
                resource_group_name=resource_group_name,
                workspace_name=workspace_name,
                name=name,
                version=version,
                body=body,
                api_version=api_version,
                content_type=content_type,
                cls=lambda x,y,z: x,
                **kwargs
            )
        kwargs.pop('error_map', None)

        def get_long_running_output(pipeline_response):
            response = pipeline_response.http_response
            deserialized = self._deserialize('FeaturesetVersion', pipeline_response)
            if cls:
                return cls(pipeline_response, deserialized, {})
            return deserialized


        if polling is True: polling_method = AsyncARMPolling(lro_delay, lro_options={'final-state-via': 'original-uri'}, **kwargs)
        elif polling is False: polling_method = AsyncNoPolling()
        else: polling_method = polling
        if cont_token:
            return AsyncLROPoller.from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output
            )
        return AsyncLROPoller(self._client, raw_result, get_long_running_output, polling_method)

    begin_create_or_update.metadata = {'url': "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/featuresets/{name}/versions/{version}"}  # type: ignore

    async def _backfill_initial(
        self,
        resource_group_name: str,
        workspace_name: str,
        name: str,
        version: str,
        body: "_models.FeaturesetVersionBackfillRequest",
        **kwargs: Any
    ) -> Optional["_models.FeaturesetVersionBackfillResponse"]:
        cls = kwargs.pop('cls', None)  # type: ClsType[Optional["_models.FeaturesetVersionBackfillResponse"]]
        error_map = {
            401: ClientAuthenticationError, 404: ResourceNotFoundError, 409: ResourceExistsError
        }
        error_map.update(kwargs.pop('error_map', {}))

        api_version = kwargs.pop('api_version', "2023-08-01-preview")  # type: str
        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]

        _json = self._serialize.body(body, 'FeaturesetVersionBackfillRequest')

        request = build_backfill_request_initial(
            subscription_id=self._config.subscription_id,
            resource_group_name=resource_group_name,
            workspace_name=workspace_name,
            name=name,
            version=version,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            template_url=self._backfill_initial.metadata['url'],
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        pipeline_response = await self._client._pipeline.run(  # pylint: disable=protected-access
            request,
            stream=False,
            **kwargs
        )
        response = pipeline_response.http_response

        if response.status_code not in [200, 202]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response, error_format=ARMErrorFormat)

        deserialized = None
        response_headers = {}
        if response.status_code == 200:
            deserialized = self._deserialize('FeaturesetVersionBackfillResponse', pipeline_response)

        if response.status_code == 202:
            response_headers['Location']=self._deserialize('str', response.headers.get('Location'))
            response_headers['Retry-After']=self._deserialize('int', response.headers.get('Retry-After'))
            

        if cls:
            return cls(pipeline_response, deserialized, response_headers)

        return deserialized

    _backfill_initial.metadata = {'url': "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/featuresets/{name}/versions/{version}/backfill"}  # type: ignore


    @distributed_trace_async
    async def begin_backfill(
        self,
        resource_group_name: str,
        workspace_name: str,
        name: str,
        version: str,
        body: "_models.FeaturesetVersionBackfillRequest",
        **kwargs: Any
    ) -> AsyncLROPoller["_models.FeaturesetVersionBackfillResponse"]:
        """Backfill.

        Backfill.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
        :type resource_group_name: str
        :param workspace_name: Name of Azure Machine Learning workspace.
        :type workspace_name: str
        :param name: Container name. This is case-sensitive.
        :type name: str
        :param version: Version identifier. This is case-sensitive.
        :type version: str
        :param body: Feature set version backfill request entity.
        :type body: ~azure.mgmt.machinelearningservices.models.FeaturesetVersionBackfillRequest
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be AsyncARMPolling. Pass in False for
         this operation to not poll, or pass in your own initialized polling object for a personal
         polling strategy.
        :paramtype polling: bool or ~azure.core.polling.AsyncPollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of AsyncLROPoller that returns either FeaturesetVersionBackfillResponse or
         the result of cls(response)
        :rtype:
         ~azure.core.polling.AsyncLROPoller[~azure.mgmt.machinelearningservices.models.FeaturesetVersionBackfillResponse]
        :raises: ~azure.core.exceptions.HttpResponseError
        """
        api_version = kwargs.pop('api_version', "2023-08-01-preview")  # type: str
        content_type = kwargs.pop('content_type', "application/json")  # type: Optional[str]
        polling = kwargs.pop('polling', True)  # type: Union[bool, AsyncPollingMethod]
        cls = kwargs.pop('cls', None)  # type: ClsType["_models.FeaturesetVersionBackfillResponse"]
        lro_delay = kwargs.pop(
            'polling_interval',
            self._config.polling_interval
        )
        cont_token = kwargs.pop('continuation_token', None)  # type: Optional[str]
        if cont_token is None:
            raw_result = await self._backfill_initial(
                resource_group_name=resource_group_name,
                workspace_name=workspace_name,
                name=name,
                version=version,
                body=body,
                api_version=api_version,
                content_type=content_type,
                cls=lambda x,y,z: x,
                **kwargs
            )
        kwargs.pop('error_map', None)

        def get_long_running_output(pipeline_response):
            response = pipeline_response.http_response
            deserialized = self._deserialize('FeaturesetVersionBackfillResponse', pipeline_response)
            if cls:
                return cls(pipeline_response, deserialized, {})
            return deserialized


        if polling is True: polling_method = AsyncARMPolling(lro_delay, lro_options={'final-state-via': 'location'}, **kwargs)
        elif polling is False: polling_method = AsyncNoPolling()
        else: polling_method = polling
        if cont_token:
            return AsyncLROPoller.from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output
            )
        return AsyncLROPoller(self._client, raw_result, get_long_running_output, polling_method)

    begin_backfill.metadata = {'url': "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.MachineLearningServices/workspaces/{workspaceName}/featuresets/{name}/versions/{version}/backfill"}  # type: ignore



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

from typing import Tuple

import pytest
import copy
from devtools_testutils import AzureRecordedTestCase, is_live
from test_utilities.utils import assert_final_job_status, get_automl_job_properties

from azure.ai.ml import MLClient
from azure.ai.ml.automl import text_classification
from azure.ai.ml.entities._inputs_outputs import Input
from azure.ai.ml.entities._job.automl.nlp import TextClassificationJob
from azure.ai.ml.operations._run_history_constants import JobStatus


@pytest.mark.automl_test
@pytest.mark.usefixtures("recorded_test")
@pytest.mark.skipif(condition=not is_live(), reason="Datasets downloaded by test are too large to record reliably")
class TestTextClassification(AzureRecordedTestCase):
    @pytest.mark.parametrize("components", [(False), (True)])
    def test_remote_run_text_classification(
        self, newsgroup: Tuple[Input, Input, str], client: MLClient, components: bool
    ) -> None:
        training_data, validation_data, target_column_name = newsgroup

        job = text_classification(
            training_data=training_data,
            validation_data=validation_data,
            target_column_name=target_column_name,
            compute="gpu-cluster",
            experiment_name="DPv2-text-classification",
            properties=get_automl_job_properties(),
        )

        # use component specific model name so that the test fails if components are not run
        if components:
            job.set_training_parameters(model_name="microsoft/deberta-base")
            job_reuse = copy.deepcopy(job)

        job.set_limits(timeout_minutes=60, max_concurrent_trials=1)
        job.set_featurization(dataset_language="eng")

        created_job = client.jobs.create_or_update(job)
        if components:
            created_job_reuse = client.jobs.create_or_update(job_reuse)
            assert_final_job_status(created_job_reuse, client, TextClassificationJob, JobStatus.COMPLETED)

        assert_final_job_status(created_job, client, TextClassificationJob, JobStatus.COMPLETED)



# ---------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# ---------------------------------------------------------

import pytest

from azure.ai.ml._restclient.v2022_05_01.models import BatchOutputAction
from azure.ai.ml.constants._deployment import BatchDeploymentOutputAction
from azure.ai.ml.entities import ModelBatchDeployment
from azure.ai.ml.entities._load_functions import load_model_batch_deployment
from azure.ai.ml.exceptions import ValidationException


@pytest.mark.unittest
class TestModelBatchDeployment:
    MODEL_BATCH_DEPLOYMNET = "./tests/test_configs/deployments/batch/model_batch_deployment.yaml"

    def test_to_rest_object(self) -> None:
        deployment = load_model_batch_deployment(
            TestModelBatchDeployment.MODEL_BATCH_DEPLOYMNET, params_override=[{"endpoint_name": "some-en-name"}]
        )
        assert deployment.type == "model"
        rest_deployment = deployment._to_rest_object(location="eastus")
        assert rest_deployment.location == "eastus"
        assert rest_deployment.properties.description == deployment.description
        assert rest_deployment.properties.environment_id == deployment.environment
        assert rest_deployment.properties.model.asset_id.name == "model-1"
        assert rest_deployment.properties.code_configuration.code_id == deployment.code_configuration.code
        assert (
            rest_deployment.properties.code_configuration.scoring_script == deployment.code_configuration.scoring_script
        )
        assert rest_deployment.properties.output_file_name == deployment.settings.output_file_name
        assert str(rest_deployment.properties.output_action) == "BatchOutputAction.APPEND_ROW"
        assert rest_deployment.properties.resources.instance_count == deployment.resources.instance_count
        assert rest_deployment.properties.retry_settings.max_retries == deployment.settings.retry_settings.max_retries
        assert (
            rest_deployment.properties.max_concurrency_per_instance == deployment.settings.max_concurrency_per_instance
        )
        assert rest_deployment.properties.environment_variables == deployment.settings.environment_variables
        assert rest_deployment.properties.compute == deployment.compute
        assert rest_deployment.properties.properties == deployment.properties
        assert rest_deployment.tags == deployment.tags

    def test_output_action_yaml_to_rest(self):
        assert (
            ModelBatchDeployment._yaml_output_action_to_rest_output_action(BatchDeploymentOutputAction.APPEND_ROW)
            == BatchOutputAction.APPEND_ROW
        )
        assert (
            ModelBatchDeployment._yaml_output_action_to_rest_output_action(BatchDeploymentOutputAction.SUMMARY_ONLY)
            == BatchOutputAction.SUMMARY_ONLY
        )

    def test_validation_fails_if_output_file_name_and_summary_output_action(self) -> None:
        deployment = load_model_batch_deployment(TestModelBatchDeployment.MODEL_BATCH_DEPLOYMNET)
        deployment.settings.output_action = BatchDeploymentOutputAction.SUMMARY_ONLY
        deployment.settings.output_file_name = "some_file_name"
        with pytest.raises(ValidationException) as ex:
            deployment._to_rest_object(location="westus")
        assert "When output_action is set to summary_only, the output_file_name need not to be specified." == str(
            ex.value
        )

    def test_to_dict(self) -> None:
        deployment = load_model_batch_deployment(TestModelBatchDeployment.MODEL_BATCH_DEPLOYMNET)
        deployment_dict = deployment._to_dict()
        assert deployment_dict["name"] == deployment.name
        assert deployment_dict["endpoint_name"] == deployment.endpoint_name
        assert deployment_dict["model"]["name"] == deployment.model.name
        assert deployment_dict["compute"] == "azureml:cpu-cluster"
        assert deployment_dict["resources"]["instance_count"] == deployment.resources.instance_count
        assert deployment_dict["settings"]["error_threshold"] == deployment.settings.error_threshold
        assert deployment_dict["settings"]["output_action"] == "append_row"

    def test_settings_getter_setter(self) -> None:
        """Test getter and setter for settings in ModelBatchDeployment."""
        # Create a minimal batch deployment
        deployment = ModelBatchDeployment(
            name="test-deployment",
            endpoint_name="test-endpoint",
            model="azureml:model:1",
            compute="azureml:cpu-cluster",
            resources={"instance_count": 2},
        )
        assert deployment.compute == "azureml:cpu-cluster"
        assert deployment.resources["instance_count"] == 2

        # Test setting individual settings properties
        deployment.settings.output_file_name = "results.csv"
        deployment.settings.output_action = BatchDeploymentOutputAction.APPEND_ROW
        deployment.settings.error_threshold = 10
        deployment.settings.max_concurrency_per_instance = 4
        deployment.settings.mini_batch_size = 5
        deployment.settings.environment_variables = {"ENV_VAR1": "value1", "ENV_VAR2": "value2"}

        # Verify individual settings can be retrieved correctly
        assert deployment.settings.output_file_name == "results.csv"
        assert deployment.settings.output_action == BatchDeploymentOutputAction.APPEND_ROW
        assert deployment.settings.error_threshold == 10
        assert deployment.settings.max_concurrency_per_instance == 4
        assert deployment.settings.mini_batch_size == 5
        assert deployment.settings.environment_variables == {"ENV_VAR1": "value1", "ENV_VAR2": "value2"}
